@@ -158,7 +158,7 @@ class UnitOfWork(object):
             if x._state not in self.deleted 
             and (
                 x._state.modified
-                or (x.__class__._class_state.has_mutable_scalars and x.state.is_modified())
+                or (x.__class__._class_state.has_mutable_scalars and x._state.is_modified())
             )
             ])
 
@@ -216,10 +216,10 @@ class UnitOfWork(object):
             
             if session.extension is not None:
                 session.extension.after_flush(session, flush_context)
+            session.commit()
         except:
             session.rollback()
             raise
-        session.commit()
 
         flush_context.post_exec()
 
@@ -766,7 +766,7 @@ class UOWTask(object):
                 # TODO: no test coverage for this !!
                 localtask = UOWTask(self.uowtransaction, t2.mapper)
                 for state in t2.elements:
-                    localtask.append(obj, t2.listonly, isdelete=t2._objects[state].isdelete)
+                    localtask.append(state, t2.listonly, isdelete=t2._objects[state].isdelete)
                 for dep in t2.dependencies:
                     localtask.dependencies.add(dep)
                 ret.insert(0, localtask)
