@@ -4,28 +4,29 @@
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
-"""defines genericized SQL types, each represented by a subclass of 
-[sqlalchemy.types#AbstractType].  Dialects define further subclasses of these 
+"""defines genericized SQL types, each represented by a subclass of
+[sqlalchemy.types#AbstractType].  Dialects define further subclasses of these
 types.
 
 For more information see the SQLAlchemy documentation on types.
 
 """
 __all__ = [ 'TypeEngine', 'TypeDecorator', 'AbstractType',
-            'INT', 'CHAR', 'VARCHAR', 'NCHAR', 'TEXT', 'FLOAT',
+            'INT', 'CHAR', 'VARCHAR', 'NCHAR', 'TEXT', 'Text', 'FLOAT',
             'NUMERIC', 'DECIMAL', 'TIMESTAMP', 'DATETIME', 'CLOB', 'BLOB',
             'BOOLEAN', 'SMALLINT', 'DATE', 'TIME',
             'String', 'Integer', 'SmallInteger','Smallinteger',
             'Numeric', 'Float', 'DateTime', 'Date', 'Time', 'Binary',
-            'Boolean', 'Unicode', 'PickleType', 'Interval',
+            'Boolean', 'Unicode', 'UnicodeText', 'PickleType', 'Interval',
+            'type_map'
             ]
 
 import inspect
 import datetime as dt
-import warnings
 
 from sqlalchemy import exceptions
 from sqlalchemy.util import pickle, Decimal as _python_Decimal
+import sqlalchemy.util as util
 NoneType = type(None)
 
 class _UserTypeAdapter(type):
@@ -155,7 +156,7 @@ class AbstractType(object):
                       for k in inspect.getargspec(self.__init__)[0][1:]]))
 
 class TypeEngine(AbstractType):
-    def dialect_impl(self, dialect):
+    def dialect_impl(self, dialect, **kwargs):
         try:
             return self._impl_dict[dialect]
         except AttributeError:
@@ -196,7 +197,7 @@ class TypeDecorator(AbstractType):
             raise exceptions.AssertionError("TypeDecorator implementations require a class-level variable 'impl' which refers to the class of type being decorated")
         self.impl = self.__class__.impl(*args, **kwargs)
 
-    def dialect_impl(self, dialect):
+    def dialect_impl(self, dialect, **kwargs):
         try:
             return self._impl_dict[dialect]
         except AttributeError:
@@ -221,7 +222,7 @@ class TypeDecorator(AbstractType):
         by default calls dialect.type_descriptor(self.impl), but
         can be overridden to provide different behavior.
         """
-        
+
         return dialect.type_descriptor(self.impl)
 
     def __getattr__(self, key):
@@ -234,10 +235,10 @@ class TypeDecorator(AbstractType):
 
     def process_bind_param(self, value, dialect):
         raise NotImplementedError()
-    
+
     def process_result_value(self, value, dialect):
         raise NotImplementedError()
-        
+
     def bind_processor(self, dialect):
         if self.__class__.process_bind_param.func_code is not TypeDecorator.process_bind_param.func_code:
             impl_processor = self.impl.bind_processor(dialect)
@@ -340,6 +341,15 @@ class Concatenable(object):
             return op
 
 class String(Concatenable, TypeEngine):
+    """A sized string type.
+
+    Usually corresponds to VARCHAR.  Can also take Python unicode objects
+    and encode to the database's encoding in bind params (and the reverse for
+    result sets.)
+
+    a String with no length will adapt itself automatically to a Text
+    object at the dialect level (this behavior is deprecated in 0.4).
+    """
     def __init__(self, length=None, convert_unicode=False, assert_unicode=None):
         self.length = length
         self.convert_unicode = convert_unicode
@@ -359,7 +369,8 @@ class String(Concatenable, TypeEngine):
                     return value.encode(dialect.encoding)
                 elif assert_unicode and not isinstance(value, (unicode, NoneType)):
                     if assert_unicode == 'warn':
-                        warnings.warn(RuntimeWarning("Unicode type received non-unicode bind param value %r" % value))
+                        util.warn("Unicode type received non-unicode bind "
+                                  "param value %r" % value)
                         return value
                     else:
                         raise exceptions.InvalidRequestError("Unicode type received non-unicode bind param value %r" % value)
@@ -380,25 +391,49 @@ class String(Concatenable, TypeEngine):
         else:
             return None
 
+    def dialect_impl(self, dialect, **kwargs):
+        _for_ddl = kwargs.pop('_for_ddl', False)
+        if _for_ddl and self.length is None:
+            label = util.to_ascii(_for_ddl is True and
+                                  '' or (' for column "%s"' % str(_for_ddl)))
+            util.warn_deprecated(
+                "Using String type with no length for CREATE TABLE "
+                "is deprecated; use the Text type explicitly" + label)
+        return TypeEngine.dialect_impl(self, dialect, **kwargs)
+
     def get_search_list(self):
         l = super(String, self).get_search_list()
         # if we are String or Unicode with no length,
-        # return TEXT as the highest-priority type
+        # return Text as the highest-priority type
         # to be adapted by the dialect
         if self.length is None and l[0] in (String, Unicode):
-            return (TEXT,) + l
+            return (Text,) + l
         else:
             return l
 
     def get_dbapi_type(self, dbapi):
         return dbapi.STRING
 
+class Text(String):
+    def dialect_impl(self, dialect, **kwargs):
+        return TypeEngine.dialect_impl(self, dialect, **kwargs)
+
 class Unicode(String):
+    """A synonym for String(length, convert_unicode=True, assert_unicode='warn')."""
+
     def __init__(self, length=None, **kwargs):
         kwargs['convert_unicode'] = True
         kwargs['assert_unicode'] = 'warn'
         super(Unicode, self).__init__(length=length, **kwargs)
 
+class UnicodeText(Text):
+    """A synonym for Text(convert_unicode=True, assert_unicode='warn')."""
+
+    def __init__(self, length=None, **kwargs):
+        kwargs['convert_unicode'] = True
+        kwargs['assert_unicode'] = 'warn'
+        super(UnicodeText, self).__init__(length=length, **kwargs)
+
 class Integer(TypeEngine):
     """Integer datatype."""
 
@@ -408,11 +443,11 @@ class Integer(TypeEngine):
 class SmallInteger(Integer):
     """Smallint datatype."""
 
-    pass
-
 Smallinteger = SmallInteger
 
 class Numeric(TypeEngine):
+    """Numeric datatype, usually resolves to DECIMAL or NUMERIC."""
+
     def __init__(self, precision=10, length=2, asdecimal=True):
         self.precision = precision
         self.length = length
@@ -634,7 +669,8 @@ class Interval(TypeDecorator):
             return process
 
 class FLOAT(Float): pass
-class TEXT(String): pass
+
+TEXT = Text
 class NUMERIC(Numeric): pass
 class DECIMAL(Numeric): pass
 class INT(Integer): pass
@@ -644,7 +680,7 @@ class TIMESTAMP(DateTime): pass
 class DATETIME(DateTime): pass
 class DATE(Date): pass
 class TIME(Time): pass
-class CLOB(TEXT): pass
+class CLOB(Text): pass
 class VARCHAR(String): pass
 class CHAR(String): pass
 class NCHAR(Unicode): pass
@@ -652,3 +688,17 @@ class BLOB(Binary): pass
 class BOOLEAN(Boolean): pass
 
 NULLTYPE = NullType()
+
+# using VARCHAR/NCHAR so that we dont get the genericized "String"
+# type which usually resolves to TEXT/CLOB
+type_map = {
+    str : VARCHAR,
+    unicode : NCHAR,
+    int : Integer,
+    float : Numeric,
+    dt.date : Date,
+    dt.datetime : DateTime,
+    dt.time : Time,
+    dt.timedelta : Interval,
+    type(None): NullType
+}
