@@ -1,20 +1,20 @@
 """tests the "bind" attribute/argument across schema, SQL, and ORM sessions,
 including the deprecated versions of these arguments"""
 
-import testbase
+import testenv; testenv.configure_for_tests()
 from sqlalchemy import *
 from sqlalchemy import engine, exceptions
 from testlib import *
 
 
-class BindTest(PersistTest):
+class BindTest(TestBase):
     def test_create_drop_explicit(self):
         metadata = MetaData()
-        table = Table('test_table', metadata,   
+        table = Table('test_table', metadata,
             Column('foo', Integer))
         for bind in (
-            testbase.db,
-            testbase.db.connect()
+            testing.db,
+            testing.db.connect()
         ):
             for args in [
                 ([], {'bind':bind}),
@@ -26,15 +26,14 @@ class BindTest(PersistTest):
                 table.create(*args[0], **args[1])
                 table.drop(*args[0], **args[1])
                 assert not table.exists(*args[0], **args[1])
-    
+
     def test_create_drop_err(self):
         metadata = MetaData()
-        table = Table('test_table', metadata,   
+        table = Table('test_table', metadata,
             Column('foo', Integer))
 
         for meth in [
             metadata.create_all,
-            table.exists,
             metadata.drop_all,
             table.create,
             table.drop,
@@ -42,18 +41,64 @@ class BindTest(PersistTest):
             try:
                 meth()
                 assert False
-            except exceptions.InvalidRequestError, e:
-                assert str(e)  == "This SchemaItem is not connected to any Engine or Connection."
-        
+            except exceptions.UnboundExecutionError, e:
+                self.assertEquals(
+                    str(e),
+                    "The MetaData "
+                    "is not bound to an Engine or Connection.  "
+                    "Execution can not proceed without a database to execute "
+                    "against.  Either execute with an explicit connection or "
+                    "assign the MetaData's .bind to enable implicit execution.")
+
+        for meth in [
+            table.exists,
+            # future:
+            #table.create,
+            #table.drop,
+        ]:
+            try:
+                meth()
+                assert False
+            except exceptions.UnboundExecutionError, e:
+                self.assertEquals(
+                    str(e),
+                    "The Table 'test_table' "
+                    "is not bound to an Engine or Connection.  "
+                    "Execution can not proceed without a database to execute "
+                    "against.  Either execute with an explicit connection or "
+                    "assign this Table's .metadata.bind to enable implicit "
+                    "execution.")
+
+    @testing.future
+    def test_create_drop_err2(self):
+        for meth in [
+            table.exists,
+            table.create,
+            table.drop,
+        ]:
+            try:
+                meth()
+                assert False
+            except exceptions.UnboundExecutionError, e:
+                self.assertEquals(
+                    str(e),
+                    "The Table 'test_table' "
+                    "is not bound to an Engine or Connection.  "
+                    "Execution can not proceed without a database to execute "
+                    "against.  Either execute with an explicit connection or "
+                    "assign this Table's .metadata.bind to enable implicit "
+                    "execution.")
+
+    @testing.uses_deprecated('//connect')
     def test_create_drop_bound(self):
-        
+
         for meta in (MetaData,ThreadLocalMetaData):
             for bind in (
-                testbase.db,
-                testbase.db.connect()
+                testing.db,
+                testing.db.connect()
             ):
                 metadata = meta()
-                table = Table('test_table', metadata,   
+                table = Table('test_table', metadata,
                 Column('foo', Integer))
                 metadata.bind = bind
                 assert metadata.bind is table.bind is bind
@@ -65,10 +110,11 @@ class BindTest(PersistTest):
                 assert not table.exists()
 
                 metadata = meta()
-                table = Table('test_table', metadata,   
+                table = Table('test_table', metadata,
                     Column('foo', Integer))
 
                 metadata.connect(bind)
+
                 assert metadata.bind is table.bind is bind
                 metadata.create_all()
                 assert table.exists()
@@ -81,8 +127,8 @@ class BindTest(PersistTest):
 
     def test_create_drop_constructor_bound(self):
         for bind in (
-            testbase.db,
-            testbase.db.connect()
+            testing.db,
+            testing.db.connect()
         ):
             try:
                 for args in (
@@ -90,7 +136,7 @@ class BindTest(PersistTest):
                     ([], {'bind':bind}),
                 ):
                     metadata = MetaData(*args[0], **args[1])
-                    table = Table('test_table', metadata,   
+                    table = Table('test_table', metadata,
                         Column('foo', Integer))
                     assert metadata.bind is table.bind is bind
                     metadata.create_all()
@@ -105,11 +151,11 @@ class BindTest(PersistTest):
 
     def test_implicit_execution(self):
         metadata = MetaData()
-        table = Table('test_table', metadata,   
+        table = Table('test_table', metadata,
             Column('foo', Integer),
             test_needs_acid=True,
             )
-        conn = testbase.db.connect()
+        conn = testing.db.connect()
         metadata.create_all(bind=conn)
         try:
             trans = conn.begin()
@@ -124,13 +170,13 @@ class BindTest(PersistTest):
             assert conn.execute("select count(1) from test_table").scalar() == 0
         finally:
             metadata.drop_all(bind=conn)
-            
+
 
     def test_clauseelement(self):
         metadata = MetaData()
-        table = Table('test_table', metadata,   
+        table = Table('test_table', metadata,
             Column('foo', Integer))
-        metadata.create_all(bind=testbase.db)
+        metadata.create_all(bind=testing.db)
         try:
             for elem in [
                 table.select,
@@ -139,8 +185,8 @@ class BindTest(PersistTest):
                 lambda **kwargs:text("select * from test_table", **kwargs)
             ]:
                 for bind in (
-                    testbase.db,
-                    testbase.db.connect()
+                    testing.db,
+                    testing.db.connect()
                 ):
                     try:
                         e = elem(bind=bind)
@@ -155,27 +201,32 @@ class BindTest(PersistTest):
                     assert e.bind is None
                     e.execute()
                     assert False
-                except exceptions.InvalidRequestError, e:
-                    assert str(e) == "This Compiled object is not bound to any Engine or Connection."
+                except exceptions.UnboundExecutionError, e:
+                    assert str(e).endswith(
+                        'is not bound to an Engine or '
+                        'Connection.  Execution can not proceed without a '
+                        'database to execute against.  Either execute with '
+                        'an explicit connection or bind the MetaData of the '
+                        'underlying tables to enable implicit execution.')
 
         finally:
             if isinstance(bind, engine.Connection):
                 bind.close()
-            metadata.drop_all(bind=testbase.db)
-    
+            metadata.drop_all(bind=testing.db)
+
     def test_session(self):
         from sqlalchemy.orm import create_session, mapper
         metadata = MetaData()
-        table = Table('test_table', metadata,   
+        table = Table('test_table', metadata,
             Column('foo', Integer, Sequence('foo_seq', optional=True), primary_key=True),
             Column('data', String(30)))
         class Foo(object):
             pass
         mapper(Foo, table)
-        metadata.create_all(bind=testbase.db)
+        metadata.create_all(bind=testing.db)
         try:
-            for bind in (testbase.db, 
-                testbase.db.connect()
+            for bind in (testing.db,
+                testing.db.connect()
                 ):
                 try:
                     for args in ({'bind':bind},):
@@ -191,7 +242,7 @@ class BindTest(PersistTest):
 
                 if isinstance(bind, engine.Connection):
                     bind.close()
-                    
+
             sess = create_session()
             f = Foo()
             sess.save(f)
@@ -203,8 +254,8 @@ class BindTest(PersistTest):
         finally:
             if isinstance(bind, engine.Connection):
                 bind.close()
-            metadata.drop_all(bind=testbase.db)
-        
-               
+            metadata.drop_all(bind=testing.db)
+
+
 if __name__ == '__main__':
-    testbase.main()
+    testenv.main()
