@@ -1,4 +1,4 @@
-import testbase
+import testenv; testenv.configure_for_tests()
 
 from sqlalchemy import *
 from sqlalchemy.orm import *
@@ -33,21 +33,21 @@ class ObjectCollection(object):
     def __iter__(self):
         return iter(self.values)
 
-class _CollectionOperations(PersistTest):
+class _CollectionOperations(TestBase):
     def setUp(self):
         collection_class = self.collection_class
 
-        metadata = MetaData(testbase.db)
+        metadata = MetaData(testing.db)
 
         parents_table = Table('Parent', metadata,
                               Column('id', Integer, primary_key=True),
-                              Column('name', String))
+                              Column('name', String(128)))
         children_table = Table('Children', metadata,
                                Column('id', Integer, primary_key=True),
                                Column('parent_id', Integer,
                                       ForeignKey('Parent.id')),
-                               Column('foo', String),
-                               Column('name', String))
+                               Column('foo', String(128)),
+                               Column('name', String(128)))
 
         class Parent(object):
             children = association_proxy('_children', 'name')
@@ -189,6 +189,49 @@ class _CollectionOperations(PersistTest):
 
         self.assertRaises(TypeError, set, [p1.children])
 
+        p1.children *= 0
+        after = []
+        self.assert_(p1.children == after)
+        self.assert_([c.name for c in p1._children] == after)
+
+        p1.children += ['a', 'b']
+        after = ['a', 'b']
+        self.assert_(p1.children == after)
+        self.assert_([c.name for c in p1._children] == after)
+
+        p1.children += ['c']
+        after = ['a', 'b', 'c']
+        self.assert_(p1.children == after)
+        self.assert_([c.name for c in p1._children] == after)
+
+        p1.children *= 1
+        after = ['a', 'b', 'c']
+        self.assert_(p1.children == after)
+        self.assert_([c.name for c in p1._children] == after)
+
+        p1.children *= 2
+        after = ['a', 'b', 'c', 'a', 'b', 'c']
+        self.assert_(p1.children == after)
+        self.assert_([c.name for c in p1._children] == after)
+
+        p1.children = ['a']
+        after = ['a']
+        self.assert_(p1.children == after)
+        self.assert_([c.name for c in p1._children] == after)
+
+        self.assert_((p1.children * 2) == ['a', 'a'])
+        self.assert_((2 * p1.children) == ['a', 'a'])
+        self.assert_((p1.children * 0) == [])
+        self.assert_((0 * p1.children) == [])
+
+        self.assert_((p1.children + ['b']) == ['a', 'b'])
+        self.assert_((['b'] + p1.children) == ['b', 'a'])
+
+        try:
+            p1.children + 123
+            assert False
+        except TypeError:
+            assert True
 
 class DefaultTest(_CollectionOperations):
     def __init__(self, *args, **kw):
@@ -275,7 +318,7 @@ class CustomDictTest(DictTest):
         self.assert_(len(p1._children) == 3)
         self.assert_(len(p1.children) == 3)
 
-        p1.children['e'] = 'changed-in-place' 
+        p1.children['e'] = 'changed-in-place'
         self.assert_(p1.children['e'] == 'changed-in-place')
         inplace_id = p1._children['e'].id
         p1 = self.roundtrip(p1)
@@ -485,6 +528,38 @@ class SetTest(_CollectionOperations):
                         print 'got', repr(p.children)
                         raise
 
+        # in-place mutations
+        for op in ('|=', '-=', '&=', '^='):
+            for base in (['a', 'b', 'c'], []):
+                for other in (set(['a','b','c']), set(['a','b','c','d']),
+                              set(['a']), set(['a','b']),
+                              set(['c','d']), set(['e', 'f', 'g']),
+                              set()):
+                    p = Parent('p')
+                    p.children = base[:]
+                    control = set(base[:])
+
+                    exec "p.children %s other" % op
+                    exec "control %s other" % op
+
+                    try:
+                        self.assert_(p.children == control)
+                    except:
+                        print 'Test %s %s %s:' % (set(base), op, other)
+                        print 'want', repr(control)
+                        print 'got', repr(p.children)
+                        raise
+
+                    p = self.roundtrip(p)
+
+                    try:
+                        self.assert_(p.children == control)
+                    except:
+                        print 'Test %s %s %s:' % (base, op, other)
+                        print 'want', repr(control)
+                        print 'got', repr(p.children)
+                        raise
+
 
 class CustomSetTest(SetTest):
     def __init__(self, *args, **kw):
@@ -516,20 +591,20 @@ class CustomObjectTest(_CollectionOperations):
         except TypeError:
             pass
 
-class ScalarTest(PersistTest):
+class ScalarTest(TestBase):
     def test_scalar_proxy(self):
-        metadata = MetaData(testbase.db)
+        metadata = MetaData(testing.db)
 
         parents_table = Table('Parent', metadata,
                               Column('id', Integer, primary_key=True),
-                              Column('name', String))
+                              Column('name', String(128)))
         children_table = Table('Children', metadata,
                                Column('id', Integer, primary_key=True),
                                Column('parent_id', Integer,
                                       ForeignKey('Parent.id')),
-                               Column('foo', String),
-                               Column('bar', String),
-                               Column('baz', String))
+                               Column('foo', String(128)),
+                               Column('bar', String(128)),
+                               Column('baz', String(128)))
 
         class Parent(object):
             foo = association_proxy('child', 'foo')
@@ -633,19 +708,19 @@ class ScalarTest(PersistTest):
         p2.bar = 'quux'
 
 
-class LazyLoadTest(PersistTest):
+class LazyLoadTest(TestBase):
     def setUp(self):
-        metadata = MetaData(testbase.db)
+        metadata = MetaData(testing.db)
 
         parents_table = Table('Parent', metadata,
                               Column('id', Integer, primary_key=True),
-                              Column('name', String))
+                              Column('name', String(128)))
         children_table = Table('Children', metadata,
                                Column('id', Integer, primary_key=True),
                                Column('parent_id', Integer,
                                       ForeignKey('Parent.id')),
-                               Column('foo', String),
-                               Column('name', String))
+                               Column('foo', String(128)),
+                               Column('name', String(128)))
 
         class Parent(object):
             children = association_proxy('_children', 'name')
@@ -741,4 +816,4 @@ class LazyLoadTest(PersistTest):
 
 
 if __name__ == "__main__":
-    testbase.main()
+    testenv.main()
