@@ -1,4 +1,4 @@
-import testbase
+import testenv; testenv.configure_for_tests()
 import operator
 from sqlalchemy import *
 from sqlalchemy.orm import *
@@ -9,8 +9,8 @@ from query import QueryTest
 
 class DynamicTest(FixtureTest):
     keep_mappers = False
-    keep_data = True
-    
+    refresh_data = True
+
     def test_basic(self):
         mapper(User, users, properties={
             'addresses':dynamic_loader(mapper(Address, addresses))
@@ -24,6 +24,29 @@ class DynamicTest(FixtureTest):
         assert [User(id=7, addresses=[Address(id=1, email_address='jack@bean.com')])] == q.filter(User.id==7).all()
         assert fixtures.user_address_result == q.all()
 
+    def test_count(self):
+        mapper(User, users, properties={
+            'addresses':dynamic_loader(mapper(Address, addresses))
+        })
+        sess = create_session()
+        u = sess.query(User).first()
+        assert u.addresses.count() == 1, u.addresses.count()
+
+    def test_backref(self):
+        mapper(Address, addresses, properties={
+            'user':relation(User, backref=backref('addresses', lazy='dynamic'))
+        })
+        mapper(User, users)
+
+        sess = create_session()
+        ad = sess.query(Address).get(1)
+        def go():
+            ad.user = None
+        self.assert_sql_count(testing.db, go, 1)
+        sess.flush()
+        u = sess.query(User).get(7)
+        assert ad not in u.addresses
+
     def test_no_count(self):
         mapper(User, users, properties={
             'addresses':dynamic_loader(mapper(Address, addresses))
@@ -35,7 +58,23 @@ class DynamicTest(FixtureTest):
         # result), else additional count() queries are issued when evaluating in a list context
         def go():
             assert [User(id=7, addresses=[Address(id=1, email_address='jack@bean.com')])] == q.filter(User.id==7).all()
-        self.assert_sql_count(testbase.db, go, 2)
+        self.assert_sql_count(testing.db, go, 2)
+
+    def test_m2m(self):
+        mapper(Order, orders, properties={
+            'items':relation(Item, secondary=order_items, lazy="dynamic", backref=backref('orders', lazy="dynamic"))
+        })
+        mapper(Item, items)
+
+        sess = create_session()
+        o1 = Order(id=15, description="order 10")
+        i1 = Item(id=10, description="item 8")
+        o1.items.append(i1)
+        sess.save(o1)
+        sess.flush()
+
+        assert o1 in i1.orders.all()
+        assert i1 in o1.items.all()
 
 class FlushTest(FixtureTest):
     def test_basic(self):
@@ -87,8 +126,14 @@ class FlushTest(FixtureTest):
         assert [Address(email_address='a'), Address(email_address='b'), Address(email_address='d')] == list(u.addresses)
 
         sess.delete(u)
+
+        # u.addresses relation will have to force the load
+        # of all addresses so that they can be updated
+        sess.flush()
         sess.close()
 
+        assert testing.db.scalar(addresses.count(addresses.c.user_id != None)) ==0
+
     @testing.fails_on('maxdb')
     def test_remove_orphans(self):
         mapper(User, users, properties={
@@ -125,6 +170,7 @@ class FlushTest(FixtureTest):
         sess.delete(u)
         sess.close()
 
+
 def create_backref_test(autoflush, saveuser):
     def test_backref(self):
         mapper(User, users, properties={
@@ -158,16 +204,66 @@ def create_backref_test(autoflush, saveuser):
             sess.flush()
         self.assert_(list(u.addresses) == [])
 
-    test_backref.__name__ = "test%s%s" % (
-        (autoflush and "_autoflush" or ""),
-        (saveuser and "_saveuser" or "_savead"),
-    )
+    test_backref = _function_named(
+        test_backref, "test%s%s" % ((autoflush and "_autoflush" or ""),
+                                    (saveuser and "_saveuser" or "_savead")))
     setattr(FlushTest, test_backref.__name__, test_backref)
 
 for autoflush in (False, True):
     for saveuser in (False, True):
         create_backref_test(autoflush, saveuser)
 
-if __name__ == '__main__':
-    testbase.main()
+class DontDereferenceTest(ORMTest):
+    def define_tables(self, metadata):
+        global users_table, addresses_table
 
+        users_table = Table('users', metadata,
+                           Column('id', Integer, primary_key=True),
+                           Column('name', String(40)),
+                           Column('fullname', String(100)),
+                           Column('password', String(15)))
+
+        addresses_table = Table('addresses', metadata,
+                                Column('id', Integer, primary_key=True),
+                                Column('email_address', String(100), nullable=False),
+                                Column('user_id', Integer, ForeignKey('users.id')))
+    def test_no_deref(self):
+        mapper(User, users_table, properties={
+            'addresses': relation(Address, backref='user', lazy='dynamic')
+            })
+
+        mapper(Address, addresses_table)
+
+        session = create_session()
+        user = User()
+        user.name = 'joe'
+        user.fullname = 'Joe User'
+        user.password = 'Joe\'s secret'
+        address = Address()
+        address.email_address = 'joe@joesdomain.example'
+        address.user = user
+        session.save(user)
+        session.flush()
+        session.clear()
+
+        def query1():
+            session = create_session(metadata.bind)
+            user = session.query(User).first()
+            return user.addresses.all()
+
+        def query2():
+            session = create_session(metadata.bind)
+            return session.query(User).first().addresses.all()
+
+        def query3():
+            session = create_session(metadata.bind)
+            user = session.query(User).first()
+            return session.query(User).first().addresses.all()
+
+        self.assertEquals(query1(), [Address(email_address='joe@joesdomain.example')]  )
+        self.assertEquals(query2(), [Address(email_address='joe@joesdomain.example')]  )
+        self.assertEquals(query3(), [Address(email_address='joe@joesdomain.example')]  )
+
+
+if __name__ == '__main__':
+    testenv.main()
