@@ -1,6 +1,6 @@
 """basic tests of eager loaded attributes"""
 
-import testbase
+import testenv; testenv.configure_for_tests()
 from sqlalchemy import *
 from sqlalchemy.orm import *
 from testlib import *
@@ -10,9 +10,6 @@ from query import QueryTest
 class EagerTest(FixtureTest):
     keep_mappers = False
     keep_data = True
-    
-    def setup_mappers(self):
-        pass
 
     def test_basic(self):
         mapper(User, users, properties={
@@ -65,15 +62,15 @@ class EagerTest(FixtureTest):
         assert [
             User(id=7, addresses=[
                 Address(id=1)
-            ]), 
+            ]),
             User(id=8, addresses=[
                 Address(id=3, email_address='ed@bettyboop.com'),
                 Address(id=4, email_address='ed@lala.com'),
                 Address(id=2, email_address='ed@wood.com')
-            ]), 
+            ]),
             User(id=9, addresses=[
                 Address(id=5)
-            ]), 
+            ]),
             User(id=10, addresses=[])
         ] == q.all()
 
@@ -142,8 +139,8 @@ class EagerTest(FixtureTest):
             assert a.user_id==7
         # assert that the eager loader added 'user_id' to the row
         # and deferred loading of that col was disabled
-        self.assert_sql_count(testbase.db, go, 0)
-        
+        self.assert_sql_count(testing.db, go, 0)
+
         # do the mapping in reverse
         # (we would have just used an "addresses" backref but the test fixtures then require the whole
         # backref to be set up, lazy loaders trigger, etc.)
@@ -164,10 +161,10 @@ class EagerTest(FixtureTest):
             assert u.addresses[0].user_id==7
         # assert that the eager loader didn't have to affect 'user_id' here
         # and that its still deferred
-        self.assert_sql_count(testbase.db, go, 1)
-        
+        self.assert_sql_count(testing.db, go, 1)
+
         clear_mappers()
-        
+
         mapper(User, users, properties={'addresses':relation(Address, lazy=False)})
         mapper(Address, addresses, properties={
             'user_id':deferred(addresses.c.user_id),
@@ -180,8 +177,8 @@ class EagerTest(FixtureTest):
         def go():
             u = sess.query(User).limit(1).get(8)
             assert User(id=8, addresses=[Address(id=2, dingalings=[Dingaling(id=1)]), Address(id=3), Address(id=4)]) == u
-        self.assert_sql_count(testbase.db, go, 1)
-        
+        self.assert_sql_count(testing.db, go, 1)
+
     def test_many_to_many(self):
 
         mapper(Keyword, keywords)
@@ -192,11 +189,15 @@ class EagerTest(FixtureTest):
         q = create_session().query(Item)
         def go():
             assert fixtures.item_keyword_result == q.all()
-        self.assert_sql_count(testbase.db, go, 1)
+        self.assert_sql_count(testing.db, go, 1)
 
         def go():
             assert fixtures.item_keyword_result[0:2] == q.join('keywords').filter(keywords.c.name == 'red').all()
-        self.assert_sql_count(testbase.db, go, 1)
+        self.assert_sql_count(testing.db, go, 1)
+
+        def go():
+            assert fixtures.item_keyword_result[0:2] == q.join('keywords', aliased=True).filter(keywords.c.name == 'red').all()
+        self.assert_sql_count(testing.db, go, 1)
 
 
     def test_eager_option(self):
@@ -210,7 +211,7 @@ class EagerTest(FixtureTest):
         def go():
             assert fixtures.item_keyword_result[0:2] == q.options(eagerload('keywords')).join('keywords').filter(keywords.c.name == 'red').all()
 
-        self.assert_sql_count(testbase.db, go, 1)
+        self.assert_sql_count(testing.db, go, 1)
 
     def test_cyclical(self):
         """test that a circular eager relationship breaks the cycle with a lazy loader"""
@@ -262,7 +263,7 @@ class EagerTest(FixtureTest):
                 User(id=10)
 
             ] == q.all()
-        self.assert_sql_count(testbase.db, go, 1)
+        self.assert_sql_count(testing.db, go, 1)
 
     def test_double_same_mappers(self):
         """tests eager loading with two relations simulatneously, from the same table, using aliases.  """
@@ -302,7 +303,7 @@ class EagerTest(FixtureTest):
                 User(id=10)
 
             ] == q.all()
-        self.assert_sql_count(testbase.db, go, 1)
+        self.assert_sql_count(testing.db, go, 1)
 
     def test_no_false_hits(self):
         """test that eager loaders don't interpret main table columns as part of their eager load."""
@@ -365,7 +366,7 @@ class EagerTest(FixtureTest):
         def go():
             l = q.filter(s.c.u2_id==User.c.id).distinct().all()
             assert fixtures.user_address_result == l
-        self.assert_sql_count(testbase.db, go, 1)
+        self.assert_sql_count(testing.db, go, 1)
 
     @testing.fails_on('maxdb')
     def test_limit_2(self):
@@ -421,6 +422,23 @@ class EagerTest(FixtureTest):
             )
         ] == l.all()
 
+    def test_limit_4(self):
+        # tests the LIMIT/OFFSET aliasing on a mapper against a select.   original issue from ticket #904
+        sel = select([users, addresses.c.email_address], users.c.id==addresses.c.user_id).alias('useralias')
+        mapper(User, sel, properties={
+            'orders':relation(Order, primaryjoin=sel.c.id==orders.c.user_id, lazy=False)
+        })
+        mapper(Order, orders)
+
+        sess = create_session()
+        self.assertEquals(sess.query(User).first(),
+            User(name=u'jack',orders=[
+                Order(address_id=1,description=u'order 1',isopen=0,user_id=7,id=1),
+                Order(address_id=1,description=u'order 3',isopen=1,user_id=7,id=3),
+                Order(address_id=None,description=u'order 5',isopen=0,user_id=7,id=5)],
+            email_address=u'jack@bean.com',id=7)
+        )
+
     def test_one_to_many_scalar(self):
         mapper(User, users, properties = dict(
             address = relation(mapper(Address, addresses), lazy=False, uselist=False)
@@ -430,7 +448,7 @@ class EagerTest(FixtureTest):
         def go():
             l = q.filter(users.c.id == 7).all()
             assert [User(id=7, address=Address(id=1))] == l
-        self.assert_sql_count(testbase.db, go, 1)
+        self.assert_sql_count(testing.db, go, 1)
 
     @testing.fails_on('maxdb')
     def test_many_to_one(self):
@@ -445,7 +463,7 @@ class EagerTest(FixtureTest):
             assert a.user is not None
             u1 = sess.query(User).get(7)
             assert a.user is u1
-        self.assert_sql_count(testbase.db, go, 1)
+        self.assert_sql_count(testing.db, go, 1)
 
 
     def test_one_and_many(self):
@@ -466,7 +484,7 @@ class EagerTest(FixtureTest):
 
         def go():
             assert fixtures.user_order_result[0:3] == l.all()
-        self.assert_sql_count(testbase.db, go, 1)
+        self.assert_sql_count(testing.db, go, 1)
 
     def test_double_with_aggregate(self):
 
@@ -496,7 +514,7 @@ class EagerTest(FixtureTest):
                 ),
                 User(id=10),
             ] == q.all()
-        self.assert_sql_count(testbase.db, go, 1)
+        self.assert_sql_count(testing.db, go, 1)
 
     def test_wide(self):
         mapper(Order, orders, properties={'items':relation(Item, secondary=order_items, lazy=False, order_by=items.c.id)})
@@ -506,7 +524,7 @@ class EagerTest(FixtureTest):
             orders = relation(Order, lazy = False),
         ))
         q = create_session().query(User)
-        l = q.select()
+        l = q.all()
         assert fixtures.user_all_result == q.all()
 
     def test_against_select(self):
@@ -541,6 +559,96 @@ class EagerTest(FixtureTest):
         l = q.filter(addresses.c.email_address == 'ed@lala.com').filter(Address.user_id==User.id)
         assert fixtures.user_address_result[1:2] == l.all()
 
+class AddEntityTest(FixtureTest):
+    keep_mappers = False
+    keep_data = True
+
+    def _assert_result(self):
+        return [
+            (
+                User(id=7,
+                    addresses=[Address(id=1)]
+                ),
+                Order(id=1,
+                    items=[Item(id=1), Item(id=2), Item(id=3)]
+                ),
+            ),
+            (
+                User(id=7,
+                    addresses=[Address(id=1)]
+                ),
+                Order(id=3,
+                    items=[Item(id=3), Item(id=4), Item(id=5)]
+                ),
+            ),
+            (
+                User(id=7,
+                    addresses=[Address(id=1)]
+                ),
+                Order(id=5,
+                    items=[Item(id=5)]
+                ),
+            ),
+            (
+                 User(id=9,
+                    addresses=[Address(id=5)]
+                ),
+                 Order(id=2,
+                    items=[Item(id=1), Item(id=2), Item(id=3)]
+                ),
+             ),
+             (
+                  User(id=9,
+                    addresses=[Address(id=5)]
+                ),
+                  Order(id=4,
+                    items=[Item(id=1), Item(id=5)]
+                ),
+              )
+        ]
+
+    def test_basic(self):
+        mapper(User, users, properties={
+            'addresses':relation(Address, lazy=False),
+            'orders':relation(Order)
+        })
+        mapper(Address, addresses)
+        mapper(Order, orders, properties={
+            'items':relation(Item, secondary=order_items, lazy=False, order_by=items.c.id)
+        })
+        mapper(Item, items)
+
+
+        sess = create_session()
+        def go():
+            ret = sess.query(User).add_entity(Order).join('orders', aliased=True).order_by(User.id).order_by(Order.id).all()
+            self.assertEquals(ret, self._assert_result())
+        self.assert_sql_count(testing.db, go, 1)
+
+    def test_options(self):
+        mapper(User, users, properties={
+            'addresses':relation(Address),
+            'orders':relation(Order)
+        })
+        mapper(Address, addresses)
+        mapper(Order, orders, properties={
+            'items':relation(Item, secondary=order_items, order_by=items.c.id)
+        })
+        mapper(Item, items)
+
+        sess = create_session()
+
+        def go():
+            ret = sess.query(User).options(eagerload('addresses')).add_entity(Order).join('orders', aliased=True).order_by(User.id).order_by(Order.id).all()
+            self.assertEquals(ret, self._assert_result())
+        self.assert_sql_count(testing.db, go, 6)
+
+        sess.clear()
+        def go():
+            ret = sess.query(User).options(eagerload('addresses')).add_entity(Order).options(eagerload('items', Order)).join('orders', aliased=True).order_by(User.id).order_by(Order.id).all()
+            self.assertEquals(ret, self._assert_result())
+        self.assert_sql_count(testing.db, go, 1)
+
 class SelfReferentialEagerTest(ORMTest):
     def define_tables(self, metadata):
         global nodes
@@ -580,7 +688,49 @@ class SelfReferentialEagerTest(ORMTest):
                 ]),
                 Node(data='n13')
             ]) == d
-        self.assert_sql_count(testbase.db, go, 1)
+        self.assert_sql_count(testing.db, go, 1)
+
+
+    def test_lazy_fallback_doesnt_affect_eager(self):
+        class Node(Base):
+            def append(self, node):
+                self.children.append(node)
+
+        mapper(Node, nodes, properties={
+            'children':relation(Node, lazy=False, join_depth=1)
+        })
+        sess = create_session()
+        n1 = Node(data='n1')
+        n1.append(Node(data='n11'))
+        n1.append(Node(data='n12'))
+        n1.append(Node(data='n13'))
+        n1.children[1].append(Node(data='n121'))
+        n1.children[1].append(Node(data='n122'))
+        n1.children[1].append(Node(data='n123'))
+        sess.save(n1)
+        sess.flush()
+        sess.clear()
+
+        # eager load with join depth 1.  when eager load of 'n1'
+        # hits the children of 'n12', no columns are present, eager loader
+        # degrades to lazy loader; fine.  but then, 'n12' is *also* in the
+        # first level of columns since we're loading the whole table.
+        # when those rows arrive, now we *can* eager load its children and an
+        # eager collection should be initialized.  essentially the 'n12' instance
+        # is present in not just two different rows but two distinct sets of columns
+        # in this result set.
+        def go():
+            allnodes = sess.query(Node).order_by(Node.data).all()
+            n12 = allnodes[2]
+            assert n12.data == 'n12'
+            print "N12 IS", id(n12)
+            print [c.data for c in n12.children]
+            assert [
+                Node(data='n121'),
+                Node(data='n122'),
+                Node(data='n123')
+            ] == list(n12.children)
+        self.assert_sql_count(testing.db, go, 1)
 
     def test_with_deferred(self):
         class Node(Base):
@@ -601,22 +751,22 @@ class SelfReferentialEagerTest(ORMTest):
 
         def go():
             assert Node(data='n1', children=[Node(data='n11'), Node(data='n12')]) == sess.query(Node).first()
-        self.assert_sql_count(testbase.db, go, 4)
-        
+        self.assert_sql_count(testing.db, go, 4)
+
         sess.clear()
 
         def go():
             assert Node(data='n1', children=[Node(data='n11'), Node(data='n12')]) == sess.query(Node).options(undefer('data')).first()
-        self.assert_sql_count(testbase.db, go, 3)
+        self.assert_sql_count(testing.db, go, 3)
 
         sess.clear()
-        
+
         def go():
             assert Node(data='n1', children=[Node(data='n11'), Node(data='n12')]) == sess.query(Node).options(undefer('data'), undefer('children.data')).first()
-        self.assert_sql_count(testbase.db, go, 1)
-        
-        
-        
+        self.assert_sql_count(testing.db, go, 1)
+
+
+
     def test_options(self):
         class Node(Base):
             def append(self, node):
@@ -647,7 +797,7 @@ class SelfReferentialEagerTest(ORMTest):
                 ]),
                 Node(data='n13')
             ]) == d
-        self.assert_sql_count(testbase.db, go, 2)
+        self.assert_sql_count(testing.db, go, 2)
 
         def go():
             d = sess.query(Node).filter_by(data='n1').options(eagerload('children.children')).first()
@@ -656,10 +806,10 @@ class SelfReferentialEagerTest(ORMTest):
         # testing only sqlite for now since the query text is slightly different on other
         # dialects
         if testing.against('sqlite'):
-            self.assert_sql(testbase.db, go, [
+            self.assert_sql(testing.db, go, [
                 (
-                    "SELECT nodes.id AS nodes_id, nodes.parent_id AS nodes_parent_id, nodes.data AS nodes_data FROM nodes WHERE nodes.data = :nodes_data ORDER BY nodes.oid  LIMIT 1 OFFSET 0",
-                    {'nodes_data': 'n1'}
+                    "SELECT nodes.id AS nodes_id, nodes.parent_id AS nodes_parent_id, nodes.data AS nodes_data FROM nodes WHERE nodes.data = :nodes_data_1 ORDER BY nodes.oid  LIMIT 1 OFFSET 0",
+                    {'nodes_data_1': 'n1'}
                 ),
             ])
 
@@ -694,7 +844,7 @@ class SelfReferentialEagerTest(ORMTest):
                 ]),
                 Node(data='n13')
             ]) == d
-        self.assert_sql_count(testbase.db, go, 3)
+        self.assert_sql_count(testing.db, go, 3)
 
 class SelfReferentialM2MEagerTest(ORMTest):
     def define_tables(self, metadata):
@@ -723,8 +873,8 @@ class SelfReferentialM2MEagerTest(ORMTest):
         })
 
         sess = create_session()
-        w1 = Widget(name='w1')
-        w2 = Widget(name='w2')
+        w1 = Widget(name=u'w1')
+        w2 = Widget(name=u'w2')
         w1.children.append(w2)
         sess.save(w1)
         sess.flush()
@@ -732,7 +882,7 @@ class SelfReferentialM2MEagerTest(ORMTest):
 
 #        l = sess.query(Widget).filter(Widget.name=='w1').all()
 #        print l
-        assert [Widget(name='w1', children=[Widget(name='w2')])] == sess.query(Widget).filter(Widget.name=='w1').all()
+        assert [Widget(name='w1', children=[Widget(name='w2')])] == sess.query(Widget).filter(Widget.name==u'w1').all()
 
 class CyclicalInheritingEagerTest(ORMTest):
     def define_tables(self, metadata):
@@ -772,6 +922,5 @@ class CyclicalInheritingEagerTest(ORMTest):
         # testing a particular endless loop condition in eager join setup
         create_session().query(SubT).all()
 
-
 if __name__ == '__main__':
-    testbase.main()
+    testenv.main()
