@@ -1,4 +1,4 @@
-import testbase
+import testenv; testenv.configure_for_tests()
 from sqlalchemy import *
 from sqlalchemy import exceptions, util
 from sqlalchemy.orm import *
@@ -57,7 +57,7 @@ class O2MTest(ORMTest):
         sess.flush()
         compare = ','.join([repr(b1), repr(b2), repr(b1.parent_foo), repr(b2.parent_foo)])
         sess.clear()
-        l = sess.query(Blub).select()
+        l = sess.query(Blub).all()
         result = ','.join([repr(l[0]), repr(l[1]), repr(l[0].parent_foo), repr(l[1].parent_foo)])
         print compare
         print result
@@ -68,29 +68,29 @@ class CascadeTest(ORMTest):
     """that cascades on polymorphic relations continue
     cascading along the path of the instance's mapper, not
     the base mapper."""
-    
+
     def define_tables(self, metadata):
         global t1, t2, t3, t4
         t1= Table('t1', metadata,
             Column('id', Integer, primary_key=True),
             Column('data', String(30))
             )
-            
+
         t2 = Table('t2', metadata,
             Column('id', Integer, primary_key=True),
             Column('t1id', Integer, ForeignKey('t1.id')),
             Column('type', String(30)),
             Column('data', String(30))
         )
-        t3 = Table('t3', metadata, 
+        t3 = Table('t3', metadata,
             Column('id', Integer, ForeignKey('t2.id'), primary_key=True),
             Column('moredata', String(30)))
-            
+
         t4 = Table('t4', metadata,
             Column('id', Integer, primary_key=True),
             Column('t3id', Integer, ForeignKey('t3.id')),
             Column('data', String(30)))
-            
+
     def test_cascade(self):
         class T1(fixtures.Base):
             pass
@@ -100,16 +100,16 @@ class CascadeTest(ORMTest):
             pass
         class T4(fixtures.Base):
             pass
-        
+
         mapper(T1, t1, properties={
             't2s':relation(T2, cascade="all")
         })
-        mapper(T2, t2, polymorphic_on=t2.c.type, polymorphic_identity='t2') 
+        mapper(T2, t2, polymorphic_on=t2.c.type, polymorphic_identity='t2')
         mapper(T3, t3, inherits=T2, polymorphic_identity='t3', properties={
             't4s':relation(T4, cascade="all")
         })
         mapper(T4, t4)
-        
+
         sess = create_session()
         t1_1 = T1(data='t1')
 
@@ -118,22 +118,22 @@ class CascadeTest(ORMTest):
 
         t1_1.t2s.append(t2_1)
         t1_1.t2s.append(t3_1)
-        
+
         t4_1 = T4(data='t4')
         t3_1.t4s.append(t4_1)
 
         sess.save(t1_1)
 
-        
+
         assert t4_1 in sess.new
         sess.flush()
-        
+
         sess.delete(t1_1)
         assert t4_1 in sess.deleted
         sess.flush()
-        
-    
-    
+
+
+
 class GetTest(ORMTest):
     def define_tables(self, metadata):
         global foo, bar, blub
@@ -190,8 +190,8 @@ class GetTest(ORMTest):
                     assert sess.query(Bar).get(b.id) == b
                     assert sess.query(Bar).get(bl.id) == bl
                     assert sess.query(Blub).get(bl.id) == bl
-                    
-                self.assert_sql_count(testbase.db, go, 0)
+
+                self.assert_sql_count(testing.db, go, 0)
             else:
                 # this is testing the 'wrong' behavior of using get()
                 # polymorphically with mappers that are not configured to be
@@ -213,9 +213,9 @@ class GetTest(ORMTest):
 
                     assert sess.query(Blub).get(bl.id) == bl
 
-                self.assert_sql_count(testbase.db, go, 3)
-            
-        test_get.__name__ = name
+                self.assert_sql_count(testing.db, go, 3)
+
+        test_get = _function_named(test_get, name)
         return test_get
 
     test_get_polymorphic = create_test(True, 'test_get_polymorphic')
@@ -308,8 +308,8 @@ class EagerLazyTest(ORMTest):
 
         sess = create_session()
         q = sess.query(Bar)
-        self.assert_(len(q.selectfirst().lazy) == 1)
-        self.assert_(len(q.selectfirst().eager) == 1)
+        self.assert_(len(q.first().lazy) == 1)
+        self.assert_(len(q.first().eager) == 1)
 
 
 class FlushTest(ORMTest):
@@ -404,21 +404,21 @@ class FlushTest(ORMTest):
 class VersioningTest(ORMTest):
     def define_tables(self, metadata):
         global base, subtable, stuff
-        base = Table('base', metadata, 
+        base = Table('base', metadata,
             Column('id', Integer, Sequence('version_test_seq', optional=True), primary_key=True ),
             Column('version_id', Integer, nullable=False),
             Column('value', String(40)),
             Column('discriminator', Integer, nullable=False)
         )
-        subtable = Table('subtable', metadata, 
+        subtable = Table('subtable', metadata,
             Column('id', None, ForeignKey('base.id'), primary_key=True),
             Column('subdata', String(50))
             )
-        stuff = Table('stuff', metadata, 
+        stuff = Table('stuff', metadata,
             Column('id', Integer, primary_key=True),
             Column('parent', Integer, ForeignKey('base.id'))
             )
-            
+
     @engines.close_open_connections
     def test_save_update(self):
         class Base(fixtures.Base):
@@ -432,24 +432,24 @@ class VersioningTest(ORMTest):
             'stuff':relation(Stuff)
         })
         mapper(Sub, subtable, inherits=Base, polymorphic_identity=2)
-        
+
         sess = create_session()
-        
+
         b1 = Base(value='b1')
         s1 = Sub(value='sub1', subdata='some subdata')
         sess.save(b1)
         sess.save(s1)
-        
+
         sess.flush()
-        
+
         sess2 = create_session()
         s2 = sess2.query(Base).get(s1.id)
         s2.subdata = 'sess2 subdata'
-        
+
         s1.subdata = 'sess1 subdata'
-        
+
         sess.flush()
-        
+
         try:
             sess2.query(Base).with_lockmode('read').get(s1.id)
             assert False
@@ -461,49 +461,49 @@ class VersioningTest(ORMTest):
             assert False
         except exceptions.ConcurrentModificationError, e:
             assert True
-        
+
         sess2.refresh(s2)
         assert s2.subdata == 'sess1 subdata'
         s2.subdata = 'sess2 subdata'
         sess2.flush()
-    
+
     def test_delete(self):
         class Base(fixtures.Base):
             pass
         class Sub(Base):
             pass
-            
+
         mapper(Base, base, polymorphic_on=base.c.discriminator, version_id_col=base.c.version_id, polymorphic_identity=1)
         mapper(Sub, subtable, inherits=Base, polymorphic_identity=2)
-        
+
         sess = create_session()
-        
+
         b1 = Base(value='b1')
         s1 = Sub(value='sub1', subdata='some subdata')
         s2 = Sub(value='sub2', subdata='some other subdata')
         sess.save(b1)
         sess.save(s1)
         sess.save(s2)
-        
+
         sess.flush()
 
         sess2 = create_session()
         s3 = sess2.query(Base).get(s1.id)
         sess2.delete(s3)
         sess2.flush()
-        
+
         s2.subdata = 'some new subdata'
         sess.flush()
-        
+
         try:
             s1.subdata = 'some new subdata'
             sess.flush()
             assert False
         except exceptions.ConcurrentModificationError, e:
             assert True
-        
 
-    
+
+
 class DistinctPKTest(ORMTest):
     """test the construction of mapper.primary_key when an inheriting relationship
     joins on a column other than primary key column."""
@@ -529,9 +529,6 @@ class DistinctPKTest(ORMTest):
 
         class Employee(Person): pass
 
-        import warnings
-        warnings.filterwarnings("error", r".*On mapper.*distinct primary key")
-
     def insert_data(self):
         person_insert = person_table.insert()
         person_insert.execute(id=1, name='alice')
@@ -557,7 +554,7 @@ class DistinctPKTest(ORMTest):
             mapper(Employee, employee_table, inherits=person_mapper, primary_key=[person_table.c.id, employee_table.c.id])
             self._do_test(True)
             assert False
-        except RuntimeWarning, e:
+        except exceptions.SAWarning, e:
             assert str(e) == "On mapper Mapper|Employee|employees, primary key column 'employees.id' is being combined with distinct primary key column 'persons.id' in attribute 'id'.  Use explicit properties to give each column its own mapped attribute name.", str(e)
 
     def test_explicit_pk(self):
@@ -588,18 +585,18 @@ class SyncCompileTest(ORMTest):
 
         _a_table = Table('a', metadata,
            Column('id', Integer, primary_key=True),
-           Column('data1', String)
+           Column('data1', String(128))
         )
 
         _b_table = Table('b', metadata,
            Column('a_id', Integer, ForeignKey('a.id'), primary_key=True),
-           Column('data2', String)
+           Column('data2', String(128))
         )
 
         _c_table = Table('c', metadata,
         #   Column('a_id', Integer, ForeignKey('b.a_id'), primary_key=True), #works
            Column('b_a_id', Integer, ForeignKey('b.a_id'), primary_key=True),
-           Column('data3', String)
+           Column('data3', String(128))
         )
 
     def test_joins(self):
@@ -650,4 +647,4 @@ class SyncCompileTest(ORMTest):
 
 
 if __name__ == "__main__":
-    testbase.main()
+    testenv.main()
