@@ -1,6 +1,6 @@
 """tests basic polymorphic mapper loading/saving, minimal relations"""
 
-import testbase
+import testenv; testenv.configure_for_tests()
 import sets
 from sqlalchemy import *
 from sqlalchemy.orm import *
@@ -27,7 +27,7 @@ class Manager(Person):
 class Boss(Manager):
     def __repr__(self):
         return "Boss %s, status %s, manager_name %s golf swing %s" % (self.get_name(), self.status, self.manager_name, self.golf_swing)
-        
+
 class Company(object):
     def __init__(self, **kwargs):
         for key, value in kwargs.iteritems():
@@ -38,38 +38,38 @@ class Company(object):
 class PolymorphTest(ORMTest):
     def define_tables(self, metadata):
         global companies, people, engineers, managers, boss
-        
+
         # a table to store companies
-        companies = Table('companies', metadata, 
+        companies = Table('companies', metadata,
            Column('company_id', Integer, Sequence('company_id_seq', optional=True), primary_key=True),
            Column('name', String(50)))
 
         # we will define an inheritance relationship between the table "people" and "engineers",
         # and a second inheritance relationship between the table "people" and "managers"
-        people = Table('people', metadata, 
+        people = Table('people', metadata,
            Column('person_id', Integer, Sequence('person_id_seq', optional=True), primary_key=True),
            Column('company_id', Integer, ForeignKey('companies.company_id')),
            Column('name', String(50)),
            Column('type', String(30)))
 
-        engineers = Table('engineers', metadata, 
+        engineers = Table('engineers', metadata,
            Column('person_id', Integer, ForeignKey('people.person_id'), primary_key=True),
            Column('status', String(30)),
            Column('engineer_name', String(50)),
            Column('primary_language', String(50)),
           )
 
-        managers = Table('managers', metadata, 
+        managers = Table('managers', metadata,
            Column('person_id', Integer, ForeignKey('people.person_id'), primary_key=True),
            Column('status', String(30)),
            Column('manager_name', String(50))
            )
 
-        boss = Table('boss', metadata, 
+        boss = Table('boss', metadata,
             Column('boss_id', Integer, ForeignKey('managers.person_id'), primary_key=True),
             Column('golf_swing', String(30)),
             )
-            
+
         metadata.create_all()
 
 class CompileTest(PolymorphTest):
@@ -88,10 +88,10 @@ class CompileTest(PolymorphTest):
         session.save(Manager(name='Tom', status='knows how to manage things'))
         session.save(Engineer(name='Kurt', status='knows how to hack'))
         session.flush()
-        print session.query(Engineer).select()
+        print session.query(Engineer).all()
+
+        print session.query(Person).all()
 
-        print session.query(Person).select()
-    
     def testcompile2(self):
         """test that a mapper can reference a property whose mapper inherits from this one."""
         person_join = polymorphic_union( {
@@ -102,7 +102,7 @@ class CompileTest(PolymorphTest):
 
 
         person_mapper = mapper(Person, people, select_table=person_join, polymorphic_on=person_join.c.type,
-                    polymorphic_identity='person', 
+                    polymorphic_identity='person',
                     properties = dict(managers = relation(Manager, lazy=True))
                 )
 
@@ -114,7 +114,7 @@ class CompileTest(PolymorphTest):
 
 class InsertOrderTest(PolymorphTest):
     def test_insert_order(self):
-        """test that classes of multiple types mix up mapper inserts 
+        """test that classes of multiple types mix up mapper inserts
         so that insert order of individual tables is maintained"""
         person_join = polymorphic_union(
             {
@@ -128,7 +128,10 @@ class InsertOrderTest(PolymorphTest):
         mapper(Engineer, engineers, inherits=person_mapper, polymorphic_identity='engineer')
         mapper(Manager, managers, inherits=person_mapper, polymorphic_identity='manager')
         mapper(Company, companies, properties={
-            'employees': relation(Person, private=True, backref='company', order_by=person_join.c.person_id)
+            'employees': relation(Person,
+                                  cascade="all, delete-orphan",
+                                  backref='company',
+                                  order_by=person_join.c.person_id)
         })
 
         session = create_session()
@@ -150,25 +153,25 @@ class InsertOrderTest(PolymorphTest):
 class RelationToSubclassTest(PolymorphTest):
     def testrelationtosubclass(self):
         """test a relation to an inheriting mapper where the relation is to a subclass
-        but the join condition is expressed by the parent table.  
-        
+        but the join condition is expressed by the parent table.
+
         also test that backrefs work in this case.
-        
+
         this test touches upon a lot of the join/foreign key determination code in properties.py
-        and creates the need for properties.py to search for conditions individually within 
+        and creates the need for properties.py to search for conditions individually within
         the mapper's local table as well as the mapper's 'mapped' table, so that relations
         requiring lots of specificity (like self-referential joins) as well as relations requiring
         more generalization (like the example here) both come up with proper results."""
-        
+
         mapper(Person, people)
-        
+
         mapper(Engineer, engineers, inherits=Person)
         mapper(Manager, managers, inherits=Person)
 
         mapper(Company, companies, properties={
             'managers': relation(Manager, lazy=True,backref="company")
         })
-        
+
         sess = create_session()
 
         c = Company(name='company1')
@@ -177,23 +180,23 @@ class RelationToSubclassTest(PolymorphTest):
         sess.flush()
         sess.clear()
 
-        sess.query(Company).get_by(company_id=c.company_id)
+        sess.query(Company).filter_by(company_id=c.company_id).one()
         assert sets.Set([e.get_name() for e in c.managers]) == sets.Set(['pointy haired boss'])
         assert c.managers[0].company is c
 
 class RoundTripTest(PolymorphTest):
     pass
-          
+
 def generate_round_trip_test(include_base=False, lazy_relation=True, redefine_colprop=False, use_literal_join=False, polymorphic_fetch=None, use_outer_joins=False):
     """generates a round trip test.
-    
+
     include_base - whether or not to include the base 'person' type in the union.
     lazy_relation - whether or not the Company relation to People is lazy or eager.
     redefine_colprop - if we redefine the 'name' column to be 'people_name' on the base Person class
     use_literal_join - primary join condition is explicitly specified
     """
     def test_roundtrip(self):
-        # create a union that represents both types of joins.  
+        # create a union that represents both types of joins.
         if not polymorphic_fetch == 'union':
             person_join = None
             manager_join = None
@@ -208,7 +211,7 @@ def generate_round_trip_test(include_base=False, lazy_relation=True, redefine_co
                         'manager':people.join(managers),
                         'person':people.select(people.c.type=='person'),
                     }, None, 'pjoin')
-                
+
                 manager_join = people.join(managers).outerjoin(boss)
         else:
             if use_outer_joins:
@@ -226,36 +229,40 @@ def generate_round_trip_test(include_base=False, lazy_relation=True, redefine_co
             person_mapper = mapper(Person, people, select_table=person_join, polymorphic_fetch=polymorphic_fetch, polymorphic_on=people.c.type, polymorphic_identity='person', properties= {'person_name':people.c.name})
         else:
             person_mapper = mapper(Person, people, select_table=person_join, polymorphic_fetch=polymorphic_fetch, polymorphic_on=people.c.type, polymorphic_identity='person')
-        
+
         mapper(Engineer, engineers, inherits=person_mapper, polymorphic_identity='engineer')
         mapper(Manager, managers, inherits=person_mapper, select_table=manager_join, polymorphic_identity='manager')
 
         mapper(Boss, boss, inherits=Manager, polymorphic_identity='boss')
-        
+
         if use_literal_join:
             mapper(Company, companies, properties={
-                'employees': relation(Person, lazy=lazy_relation, primaryjoin=people.c.company_id==companies.c.company_id, private=True, 
-                backref="company"
+                'employees': relation(Person, lazy=lazy_relation,
+                                      primaryjoin=(people.c.company_id ==
+                                                   companies.c.company_id),
+                                      cascade="all,delete-orphan",
+                                      backref="company"
                 )
             })
         else:
             mapper(Company, companies, properties={
-                'employees': relation(Person, lazy=lazy_relation, private=True, 
+                'employees': relation(Person, lazy=lazy_relation,
+                                      cascade="all, delete-orphan",
                 backref="company"
                 )
             })
-        
+
         if redefine_colprop:
             person_attribute_name = 'person_name'
         else:
             person_attribute_name = 'name'
-    
+
         session = create_session()
         c = Company(name='company1')
         c.employees.append(Manager(status='AAB', manager_name='manager1', **{person_attribute_name:'pointy haired boss'}))
         c.employees.append(Engineer(status='BBA', engineer_name='engineer1', primary_language='java', **{person_attribute_name:'dilbert'}))
         dilbert = c.employees[-1]
-        
+
         if include_base:
             c.employees.append(Person(status='HHH', **{person_attribute_name:'joesmith'}))
         c.employees.append(Engineer(status='CGG', engineer_name='engineer2', primary_language='python', **{person_attribute_name:'wally'}))
@@ -264,15 +271,15 @@ def generate_round_trip_test(include_base=False, lazy_relation=True, redefine_co
         print session.new
         session.flush()
         session.clear()
-        
+
         dilbert = session.query(Person).get(dilbert.person_id)
         assert getattr(dilbert, person_attribute_name) == 'dilbert'
         session.clear()
-        
+
         dilbert = session.query(Person).filter(Person.person_id==dilbert.person_id).one()
         assert getattr(dilbert, person_attribute_name) == 'dilbert'
         session.clear()
-        
+
         id = c.company_id
         def go():
             c = session.query(Company).get(id)
@@ -287,15 +294,15 @@ def generate_round_trip_test(include_base=False, lazy_relation=True, redefine_co
 
         if not lazy_relation:
             if polymorphic_fetch=='union':
-                self.assert_sql_count(testbase.db, go, 1)
+                self.assert_sql_count(testing.db, go, 1)
             else:
-                self.assert_sql_count(testbase.db, go, 5)
-                
+                self.assert_sql_count(testing.db, go, 5)
+
         else:
             if polymorphic_fetch=='union':
-                self.assert_sql_count(testbase.db, go, 2)
+                self.assert_sql_count(testing.db, go, 2)
             else:
-                self.assert_sql_count(testbase.db, go, 6)
+                self.assert_sql_count(testing.db, go, 6)
 
         # test selecting from the query, using the base mapped table (people) as the selection criterion.
         # in the case of the polymorphic Person query, the "people" selectable should be adapted to be "person_join"
@@ -311,10 +318,10 @@ def generate_round_trip_test(include_base=False, lazy_relation=True, redefine_co
         assert dilbert is dilbert2
 
         session.query(Person).filter((Engineer.engineer_name=="engineer1") & (Engineer.person_id==people.c.person_id)).first()
-        
+
         dilbert2 = session.query(Engineer).filter(Engineer.engineer_name=="engineer1")[0]
         assert dilbert is dilbert2
-    
+
         dilbert.engineer_name = 'hes dibert!'
 
         session.flush()
@@ -327,22 +334,21 @@ def generate_round_trip_test(include_base=False, lazy_relation=True, redefine_co
         session.clear()
         c = session.query(Manager).all()
         assert sets.Set([repr(x) for x in c]) == sets.Set(["Manager pointy haired boss, status AAB, manager_name manager1", "Manager jsmith, status ABA, manager_name manager2", "Boss daboss, status BBB, manager_name boss golf swing fore"]), repr([repr(x) for x in c])
-        
+
         c = session.query(Company).get(id)
         for e in c.employees:
             print e, e._instance_key
 
         session.delete(c)
         session.flush()
-        
-        
-    test_roundtrip.__name__ = "test_%s%s%s%s%s" % (
-        (lazy_relation and "lazy" or "eager"),
-        (include_base and "_inclbase" or ""),
-        (redefine_colprop and "_redefcol" or ""),
-        (polymorphic_fetch != 'union' and '_' + polymorphic_fetch or (use_literal_join and "_litjoin" or "")),
-        (use_outer_joins and '_outerjoins' or '')
-    )
+
+    test_roundtrip = _function_named(
+        test_roundtrip, "test_%s%s%s%s%s" % (
+          (lazy_relation and "lazy" or "eager"),
+          (include_base and "_inclbase" or ""),
+          (redefine_colprop and "_redefcol" or ""),
+          (polymorphic_fetch != 'union' and '_' + polymorphic_fetch or (use_literal_join and "_litjoin" or "")),
+          (use_outer_joins and '_outerjoins' or '')))
     setattr(RoundTripTest, test_roundtrip.__name__, test_roundtrip)
 
 for include_base in [True, False]:
@@ -355,7 +361,6 @@ for include_base in [True, False]:
                             generate_round_trip_test(include_base, lazy_relation, redefine_colprop, use_literal_join, polymorphic_fetch, use_outer_joins)
                     else:
                         generate_round_trip_test(include_base, lazy_relation, redefine_colprop, use_literal_join, polymorphic_fetch, False)
-                        
-if __name__ == "__main__":    
-    testbase.main()
 
+if __name__ == "__main__":
+    testenv.main()
