@@ -1,6 +1,6 @@
 """tests general mapper operations with an emphasis on selecting/loading"""
 
-import testbase
+import testenv; testenv.configure_for_tests()
 from sqlalchemy import *
 from sqlalchemy import exceptions, sql
 from sqlalchemy.orm import *
@@ -10,7 +10,7 @@ from testlib.tables import *
 import testlib.tables as tables
 
 
-class MapperSuperTest(AssertMixin):
+class MapperSuperTest(TestBase, AssertsExecutionResults):
     def setUpAll(self):
         tables.create()
         tables.data()
@@ -62,19 +62,18 @@ class MapperTest(MapperSuperTest):
         s = create_session()
         u = s.get(User, 7)
         assert u._user_name=='jack'
-    	assert u._user_id ==7
+        assert u._user_id ==7
         u2 = s.query(User).filter_by(user_name='jack').one()
         assert u is u2
 
-    
     def test_no_pks(self):
         s = select([users.c.user_name]).alias('foo')
         try:
             mapper(User, s)
             assert False
         except exceptions.ArgumentError, e:
-            assert str(e) == "Could not assemble any primary key columns for mapped table 'foo'"
-        
+            assert "could not assemble any primary key columns for mapped table 'foo'" in str(e)
+
     def test_compileonsession(self):
         m = mapper(User, users)
         session = create_session()
@@ -109,6 +108,7 @@ class MapperTest(MapperSuperTest):
         except TypeError, e:
             pass
 
+    @testing.uses_deprecated('SessionContext', 'SessionContextExt')
     def test_constructorexceptions(self):
         """test that exceptions raised in the mapped class are not masked by sa decorations"""
         ex = AssertionError('oops')
@@ -130,18 +130,15 @@ class MapperTest(MapperSuperTest):
         def bad_expunge(foo):
             raise Exception("this exception should be stated as a warning")
 
-        import warnings
-        warnings.filterwarnings("always", r".*this exception should be stated as a warning")
-
         sess.expunge = bad_expunge
         try:
             Foo(_sa_session=sess)
             assert False
         except Exception, e:
-            assert e is ex
+            assert isinstance(e, exceptions.SAWarning)
 
         clear_mappers()
-        
+
         # test that TypeError is raised for illegal constructor args,
         # whether or not explicit __init__ is present [ticket:908]
         class Foo(object):
@@ -149,7 +146,7 @@ class MapperTest(MapperSuperTest):
                 pass
         class Bar(object):
             pass
-                
+
         mapper(Foo, users)
         mapper(Bar, addresses)
         try:
@@ -191,17 +188,17 @@ class MapperTest(MapperSuperTest):
 
     def test_compileon_getprops(self):
         m =mapper(User, users)
-        
+
         assert not m.compiled
         assert list(m.iterate_properties)
         assert m.compiled
         clear_mappers()
-        
+
         m= mapper(User, users)
         assert not m.compiled
         assert m.get_property('user_name')
         assert m.compiled
-        
+
     def test_add_property(self):
         assert_col = []
         class User(object):
@@ -212,7 +209,7 @@ class MapperTest(MapperSuperTest):
                 assert_col.append(('set', name))
                 self._user_name = name
             user_name = property(_get_user_name, _set_user_name)
-        
+
         m = mapper(User, users)
         mapper(Address, addresses)
         m.add_property('_user_name', deferred(users.c.user_name))
@@ -221,14 +218,14 @@ class MapperTest(MapperSuperTest):
 
         sess = create_session(transactional=True)
         assert sess.query(User).get(7)
-        
+
         u = sess.query(User).filter_by(user_name='jack').one()
 
         def go():
             self.assert_result([u], User, user_address_result[0])
             assert u.user_name == 'jack'
             assert assert_col == [('get', 'jack')], str(assert_col)
-        self.assert_sql_count(testbase.db, go, 2)
+        self.assert_sql_count(testing.db, go, 2)
 
         u.name = 'ed'
         u3 = User()
@@ -236,31 +233,84 @@ class MapperTest(MapperSuperTest):
         sess.save(u3)
         sess.flush()
         sess.rollback()
-        
+
     def test_replace_property(self):
         m = mapper(User, users)
         m.add_property('_user_name',users.c.user_name)
         m.add_property('user_name', synonym('_user_name', proxy=True))
-        
+
         sess = create_session()
         u = sess.query(User).filter_by(user_name='jack').one()
         assert u._user_name == 'jack'
         assert u.user_name == 'jack'
         u.user_name = 'jacko'
         assert m._columntoproperty[users.c.user_name] is m.get_property('_user_name')
-        
+
         clear_mappers()
 
         m = mapper(User, users)
         m.add_property('user_name', synonym('_user_name', map_column=True))
-        
+
         sess.clear()
         u = sess.query(User).filter_by(user_name='jack').one()
         assert u._user_name == 'jack'
         assert u.user_name == 'jack'
         u.user_name = 'jacko'
         assert m._columntoproperty[users.c.user_name] is m.get_property('_user_name')
-        
+
+    def test_synonym_replaces_backref(self):
+        assert_calls = []
+        class Address(object):
+            def _get_user(self):
+                assert_calls.append("get")
+                return self._user
+            def _set_user(self, user):
+                assert_calls.append("set")
+                self._user = user
+            user = property(_get_user, _set_user)
+
+        # synonym is created against nonexistent prop
+        mapper(Address, addresses, properties={
+            'user':synonym('_user')
+        })
+        compile_mappers()
+
+        # later, backref sets up the prop
+        mapper(User, users, properties={
+            'addresses':relation(Address, backref='_user')
+        })
+
+        sess = create_session()
+        u1 = sess.query(User).get(7)
+        u2 = sess.query(User).get(8)
+        # comparaison ops need to work
+        a1 = sess.query(Address).filter(Address.user==u1).one()
+        assert a1.address_id == 1
+        a1.user = u2
+        assert a1.user is u2
+        self.assertEquals(assert_calls, ["set", "get"])
+
+    def test_self_ref_syn(self):
+        t = Table('nodes', MetaData(),
+            Column('id', Integer, primary_key=True),
+            Column('parent_id', Integer, ForeignKey('nodes.id')))
+
+        class Node(object):
+            pass
+
+        mapper(Node, t, properties={
+            '_children':relation(Node, backref=backref('_parent', remote_side=t.c.id)),
+            'children':synonym('_children'),
+            'parent':synonym('_parent')
+        })
+
+        n1 = Node()
+        n2 = Node()
+        n1.children.append(n2)
+        assert n2.parent is n2._parent is n1
+        assert n1.children[0] is n1._children[0] is n2
+        self.assertEquals(str(Node.parent == n2), ":param_1 = nodes.parent_id")
+
     def test_illegal_non_primary(self):
         mapper(User, users)
         mapper(Address, addresses)
@@ -271,12 +321,12 @@ class MapperTest(MapperSuperTest):
             assert False
         except exceptions.ArgumentError, e:
             assert "Attempting to assign a new relation 'addresses' to a non-primary mapper on class 'User'" in str(e)
-        
+
     def test_propfilters(self):
         t = Table('person', MetaData(),
                   Column('id', Integer, primary_key=True),
-                  Column('type', String),
-                  Column('name', String),
+                  Column('type', String(128)),
+                  Column('name', String(128)),
                   Column('employee_number', Integer),
                   Column('boss_id', Integer, ForeignKey('person.id')),
                   Column('vendor_id', Integer))
@@ -323,7 +373,8 @@ class MapperTest(MapperSuperTest):
         assert_props(Hoho, ['id', 'name', 'type'])
         assert_props(Lala, ['p_employee_number', 'p_id', 'p_name', 'p_type'])
 
-    def test_recursiveselectby(self):
+    @testing.uses_deprecated('//select_by', '//join_via', '//list')
+    def test_recursive_select_by_deprecated(self):
         """test that no endless loop occurs when traversing for select_by"""
         m = mapper(User, users, properties={
             'orders':relation(mapper(Order, orders), backref='user'),
@@ -337,7 +388,7 @@ class MapperTest(MapperSuperTest):
         usersaddresses = sql.join(users, addresses, users.c.user_id == addresses.c.user_id)
         m = mapper(User, usersaddresses, primary_key=[users.c.user_id])
         q = create_session().query(m)
-        l = q.select()
+        l = q.all()
         self.assert_result(l, User, *user_result[0:2])
 
     def test_mappingtojoinnopk(self):
@@ -353,16 +404,16 @@ class MapperTest(MapperSuperTest):
         m.compile()
         assert account_ids_table in m._pks_by_table
         assert account_stuff_table not in m._pks_by_table
-        metadata.create_all(testbase.db)
+        metadata.create_all(testing.db)
         try:
-            sess = create_session(bind=testbase.db)
+            sess = create_session(bind=testing.db)
             a = A()
             sess.save(a)
             sess.flush()
-            assert testbase.db.execute(account_ids_table.count()).scalar() == 1
-            assert testbase.db.execute(account_stuff_table.count()).scalar() == 0
+            assert testing.db.execute(account_ids_table.count()).scalar() == 1
+            assert testing.db.execute(account_stuff_table.count()).scalar() == 0
         finally:
-            metadata.drop_all(testbase.db)
+            metadata.drop_all(testing.db)
 
     def test_mappingtoouterjoin(self):
         """test mapping to an outer join, with a composite primary key that allows nulls"""
@@ -377,11 +428,27 @@ class MapperTest(MapperSuperTest):
         j = join(users, addresses, isouter=True)
         m = mapper(User, j, allow_null_pks=True, primary_key=[users.c.user_id, addresses.c.address_id])
         q = create_session().query(m)
-        l = q.select()
+        l = q.all()
         self.assert_result(l, User, *result)
 
 
     def test_customjoin(self):
+        """Tests that select_from totally replace the FROM parameters."""
+
+        m = mapper(User, users, properties={
+            'orders':relation(mapper(Order, orders, properties={
+                'items':relation(mapper(Item, orderitems))
+            }))
+        })
+
+        q = create_session().query(m)
+        l = (q.select_from(users.join(orders).join(orderitems)).
+             filter(orderitems.c.item_name=='item 4'))
+
+        self.assert_result(l, User, user_result[0])
+
+    @testing.uses_deprecated('//select')
+    def test_customjoin_deprecated(self):
         """test that the from_obj parameter to query.select() can be used
         to totally replace the FROM parameters of the generated query."""
 
@@ -397,25 +464,31 @@ class MapperTest(MapperSuperTest):
 
     def test_orderby(self):
         """test ordering at the mapper and query level"""
-        # TODO: make a unit test out of these various combinations
-#        m = mapper(User, users, order_by=desc(users.c.user_name))
-        mapper(User, users, order_by=None)
-#        mapper(User, users)
 
-#        l = create_session().query(User).select(order_by=[desc(users.c.user_name), asc(users.c.user_id)])
-        l = create_session().query(User).select()
-#        l = create_session().query(User).select(order_by=[])
-#        l = create_session().query(User).select(order_by=None)
+        # TODO: make a unit test out of these various combinations
+        #m = mapper(User, users, order_by=desc(users.c.user_name))
+        mapper(User, users, order_by=None)
+        #mapper(User, users)
+
+        #l = create_session().query(User).select(order_by=[desc(users.c.user_name), asc(users.c.user_id)])
+        l = create_session().query(User).all()
+        #l = create_session().query(User).select(order_by=[])
+        #l = create_session().query(User).select(order_by=None)
 
 
     @testing.unsupported('firebird')
     def test_function(self):
-        """test mapping to a SELECT statement that has functions in it."""
-        s = select([users, (users.c.user_id * 2).label('concat'), func.count(addresses.c.address_id).label('count')],
-        users.c.user_id==addresses.c.user_id, group_by=[c for c in users.c]).alias('myselect')
+        """Test mapping to a SELECT statement that has functions in it."""
+
+        s = select([users,
+                    (users.c.user_id * 2).label('concat'),
+                    func.count(addresses.c.address_id).label('count')],
+                   users.c.user_id == addresses.c.user_id,
+                   group_by=[c for c in users.c]).alias('myselect')
+
         mapper(User, s)
         sess = create_session()
-        l = sess.query(User).select()
+        l = sess.query(User).all()
         for u in l:
             print "User", u.user_id, u.user_name, u.concat, u.count
         assert l[0].concat == l[0].user_id * 2 == 14
@@ -430,6 +503,12 @@ class MapperTest(MapperSuperTest):
         q = create_session().query(User)
         self.assert_(q.count()==3)
         self.assert_(q.count(users.c.user_id.in_([8,9]))==2)
+
+    @testing.unsupported('firebird')
+    @testing.uses_deprecated('//count_by', '//join_by', '//join_via')
+    def test_count_by_deprecated(self):
+        mapper(User, users)
+        q = create_session().query(User)
         self.assert_(q.count_by(user_name='fred')==1)
 
     def test_manytomany_count(self):
@@ -490,11 +569,11 @@ class MapperTest(MapperSuperTest):
 
         u = sess.query(User).filter(User.uname=='jack').one()
         self.assert_result(u.adlist, Address, *(user_address_result[0]['addresses'][1]))
-        
-        addr = sess.query(Address).get_by(address_id=user_address_result[0]['addresses'][1][0]['address_id'])
+
+        addr = sess.query(Address).filter_by(address_id=user_address_result[0]['addresses'][1][0]['address_id']).one()
         u = sess.query(User).filter_by(adname=addr).one()
         u2 = sess.query(User).filter_by(adlist=addr).one()
-        
+
         assert u is u2
 
         assert u not in sess.dirty
@@ -506,12 +585,12 @@ class MapperTest(MapperSuperTest):
         assert u.user_name == "some user name"
         assert u in sess.dirty
 
-    
+
     def test_column_synonyms(self):
         """test new-style synonyms which automatically instrument properties, set up aliased column, etc."""
 
         sess = create_session()
-        
+
         assert_col = []
         class User(object):
             def _get_user_name(self):
@@ -532,27 +611,27 @@ class MapperTest(MapperSuperTest):
             assert False
         except exceptions.ArgumentError, e:
             assert str(e) == "Can't compile synonym '_user_name': no column on table 'users' named 'not_user_name'"
-        
+
         clear_mappers()
-        
+
         mapper(Address, addresses)
         mapper(User, users, properties = {
             'addresses':relation(Address, lazy=True),
             'user_name':synonym('_user_name', map_column=True)
         })
-        
+
         # test compile
         assert not isinstance(User.user_name == 'jack', bool)
-        
+
         assert hasattr(User, 'user_name')
         assert hasattr(User, '_user_name')
-        
+
         u = sess.query(User).filter(User.user_name == 'jack').one()
         assert u.user_name == 'jack'
         u.user_name = 'foo'
         assert u.user_name == 'foo'
         assert assert_col == [('get', 'jack'), ('set', 'foo'), ('get', 'foo')]
-        
+
 class OptionsTest(MapperSuperTest):
     @testing.fails_on('maxdb')
     def test_synonymoptions(self):
@@ -563,11 +642,12 @@ class OptionsTest(MapperSuperTest):
         ))
 
         def go():
-            u = sess.query(User).options(eagerload('adlist')).get_by(user_name='jack')
+            u = sess.query(User).options(eagerload('adlist')).filter_by(user_name='jack').one()
             self.assert_result(u.adlist, Address, *(user_address_result[0]['addresses'][1]))
-        self.assert_sql_count(testbase.db, go, 1)
+        self.assert_sql_count(testing.db, go, 1)
 
-    def test_extensionoptions(self):
+    @testing.uses_deprecated('//select_by')
+    def test_extension_options(self):
         sess  = create_session()
         class ext1(MapperExtension):
             def populate_instance(self, mapper, selectcontext, row, instance, **flags):
@@ -604,7 +684,7 @@ class OptionsTest(MapperSuperTest):
 
         def go():
             self.assert_result(l, User, *user_address_result)
-        self.assert_sql_count(testbase.db, go, 0)
+        self.assert_sql_count(testing.db, go, 0)
 
     @testing.fails_on('maxdb')
     def test_eageroptionswithlimit(self):
@@ -617,16 +697,16 @@ class OptionsTest(MapperSuperTest):
         def go():
             assert u.user_id == 8
             assert len(u.addresses) == 3
-        self.assert_sql_count(testbase.db, go, 0)
+        self.assert_sql_count(testing.db, go, 0)
 
         sess.clear()
 
         # test that eager loading doesnt modify parent mapper
         def go():
-            u = sess.query(User).get_by(user_id=8)
+            u = sess.query(User).filter_by(user_id=8).one()
             assert u.user_id == 8
             assert len(u.addresses) == 3
-        assert "tbl_row_count" not in self.capture_sql(testbase.db, go)
+        assert "tbl_row_count" not in self.capture_sql(testing.db, go)
 
     @testing.fails_on('maxdb')
     def test_lazyoptionswithlimit(self):
@@ -639,7 +719,7 @@ class OptionsTest(MapperSuperTest):
         def go():
             assert u.user_id == 8
             assert len(u.addresses) == 3
-        self.assert_sql_count(testbase.db, go, 1)
+        self.assert_sql_count(testing.db, go, 1)
 
     def test_eagerdegrade(self):
         """tests that an eager relation automatically degrades to a lazy relation if eager columns are not available"""
@@ -650,9 +730,9 @@ class OptionsTest(MapperSuperTest):
 
         # first test straight eager load, 1 statement
         def go():
-            l = sess.query(usermapper).select()
+            l = sess.query(usermapper).all()
             self.assert_result(l, User, *user_address_result)
-        self.assert_sql_count(testbase.db, go, 1)
+        self.assert_sql_count(testing.db, go, 1)
 
         sess.clear()
 
@@ -663,7 +743,7 @@ class OptionsTest(MapperSuperTest):
             r = users.select().execute()
             l = sess.query(usermapper).instances(r)
             self.assert_result(l, User, *user_address_result)
-        self.assert_sql_count(testbase.db, go, 4)
+        self.assert_sql_count(testing.db, go, 4)
 
         clear_mappers()
 
@@ -690,7 +770,7 @@ class OptionsTest(MapperSuperTest):
         def go():
             l = sess.query(usermapper).all()
             self.assert_result(l, User, *user_all_result)
-        self.assert_sql_count(testbase.db, go, 1)
+        self.assert_sql_count(testing.db, go, 1)
 
         sess.clear()
 
@@ -700,7 +780,7 @@ class OptionsTest(MapperSuperTest):
             r = users.select().execute()
             l = sess.query(usermapper).instances(r)
             self.assert_result(l, User, *user_all_result)
-        self.assert_sql_count(testbase.db, go, 7)
+        self.assert_sql_count(testing.db, go, 7)
 
 
     def test_lazyoptions(self):
@@ -709,10 +789,10 @@ class OptionsTest(MapperSuperTest):
         mapper(User, users, properties = dict(
             addresses = relation(mapper(Address, addresses), lazy=False)
         ))
-        l = sess.query(User).options(lazyload('addresses')).select()
+        l = sess.query(User).options(lazyload('addresses')).all()
         def go():
             self.assert_result(l, User, *user_address_result)
-        self.assert_sql_count(testbase.db, go, 3)
+        self.assert_sql_count(testing.db, go, 3)
 
     def test_deepoptions(self):
         mapper(User, users,
@@ -727,33 +807,33 @@ class OptionsTest(MapperSuperTest):
         sess = create_session()
 
         # eagerload nothing.
-        u = sess.query(User).select()
+        u = sess.query(User).all()
         def go():
             print u[0].orders[1].items[0].keywords[1]
-        self.assert_sql_count(testbase.db, go, 3)
+        self.assert_sql_count(testing.db, go, 3)
         sess.clear()
 
 
         print "-------MARK----------"
         # eagerload orders.items.keywords; eagerload_all() implies eager load of orders, orders.items
         q2 = sess.query(User).options(eagerload_all('orders.items.keywords'))
-        u = q2.select()
+        u = q2.all()
         def go():
             print u[0].orders[1].items[0].keywords[1]
         print "-------MARK2----------"
-        self.assert_sql_count(testbase.db, go, 0)
+        self.assert_sql_count(testing.db, go, 0)
 
         sess.clear()
 
         # same thing, with separate options calls
         q2 = sess.query(User).options(eagerload('orders')).options(eagerload('orders.items')).options(eagerload('orders.items.keywords'))
-        u = q2.select()
+        u = q2.all()
         def go():
             print u[0].orders[1].items[0].keywords[1]
-        self.assert_sql_count(testbase.db, go, 0)
+        self.assert_sql_count(testing.db, go, 0)
 
         sess.clear()
-        
+
         try:
             sess.query(User).options(eagerload('items', Order))
             assert False
@@ -764,8 +844,8 @@ class OptionsTest(MapperSuperTest):
         # the "items" on the order, but on "items" it will eager load the "keywords"
         print "-------MARK5----------"
         q3 = sess.query(User).options(eagerload('orders.items.keywords'))
-        u = q3.select()
-        self.assert_sql_count(testbase.db, go, 2)
+        u = q3.all()
+        self.assert_sql_count(testing.db, go, 2)
 
 
 class DeferredTest(MapperSuperTest):
@@ -782,12 +862,12 @@ class DeferredTest(MapperSuperTest):
 
         q = create_session().query(m)
         def go():
-            l = q.select()
+            l = q.all()
             o2 = l[2]
             print o2.description
 
-        orderby = str(orders.default_order_by()[0].compile(bind=testbase.db))
-        self.assert_sql(testbase.db, go, [
+        orderby = str(orders.default_order_by()[0].compile(bind=testing.db))
+        self.assert_sql(testing.db, go, [
             ("SELECT orders.order_id AS orders_order_id, orders.user_id AS orders_user_id, orders.isopen AS orders_isopen FROM orders ORDER BY %s" % orderby, {}),
             ("SELECT orders.description AS orders_description FROM orders WHERE orders.order_id = :param_1", {'param_1':3})
         ])
@@ -804,7 +884,7 @@ class DeferredTest(MapperSuperTest):
         o.order_id = 7
         def go():
             o.description = "some description"
-        self.assert_sql_count(testbase.db, go, 0)
+        self.assert_sql_count(testing.db, go, 0)
 
     def test_unsavedgroup(self):
         """test that deferred loading doesnt kick in when just PK cols are set"""
@@ -819,7 +899,7 @@ class DeferredTest(MapperSuperTest):
         o.order_id = 7
         def go():
             o.description = "some description"
-        self.assert_sql_count(testbase.db, go, 0)
+        self.assert_sql_count(testing.db, go, 0)
 
     def test_save(self):
         m = mapper(Order, orders, properties={
@@ -828,7 +908,7 @@ class DeferredTest(MapperSuperTest):
 
         sess = create_session()
         q = sess.query(m)
-        l = q.select()
+        l = q.all()
         o2 = l[2]
         o2.isopen = 1
         sess.flush()
@@ -843,30 +923,30 @@ class DeferredTest(MapperSuperTest):
         sess = create_session()
         q = sess.query(m)
         def go():
-            l = q.select()
+            l = q.all()
             o2 = l[2]
             print o2.opened, o2.description, o2.userident
             assert o2.opened == 1
             assert o2.userident == 7
             assert o2.description == 'order 3'
-        orderby = str(orders.default_order_by()[0].compile(testbase.db))
-        self.assert_sql(testbase.db, go, [
+        orderby = str(orders.default_order_by()[0].compile(testing.db))
+        self.assert_sql(testing.db, go, [
             ("SELECT orders.order_id AS orders_order_id FROM orders ORDER BY %s" % orderby, {}),
             ("SELECT orders.user_id AS orders_user_id, orders.description AS orders_description, orders.isopen AS orders_isopen FROM orders WHERE orders.order_id = :param_1", {'param_1':3})
         ])
 
-        o2 = q.select()[2]
+        o2 = q.all()[2]
 #        assert o2.opened == 1
         assert o2.description == 'order 3'
         assert o2 not in sess.dirty
         o2.description = 'order 3'
         def go():
             sess.flush()
-        self.assert_sql_count(testbase.db, go, 0)
-    
+        self.assert_sql_count(testing.db, go, 0)
+
     def test_preserve_changes(self):
         """test that the deferred load operation doesn't revert modifications on attributes"""
-        
+
         mapper(Order, orders, properties = {
             'userident':deferred(orders.c.user_id, group='primary'),
             'description':deferred(orders.c.description, group='primary'),
@@ -879,11 +959,11 @@ class DeferredTest(MapperSuperTest):
         assert o.description == 'somenewdescription'
         def go():
             assert o.opened == 1
-        self.assert_sql_count(testbase.db, go, 1)
+        self.assert_sql_count(testing.db, go, 1)
         assert o.description == 'somenewdescription'
         assert o in sess.dirty
-        
-        
+
+
     def test_commitsstate(self):
         """test that when deferred elements are loaded via a group, they get the proper CommittedState
         and dont result in changes being committed"""
@@ -895,7 +975,7 @@ class DeferredTest(MapperSuperTest):
         })
         sess = create_session()
         q = sess.query(m)
-        o2 = q.select()[2]
+        o2 = q.all()[2]
         # this will load the group of attributes
         assert o2.description == 'order 3'
         assert o2 not in sess.dirty
@@ -904,7 +984,7 @@ class DeferredTest(MapperSuperTest):
         def go():
             # therefore the flush() shouldnt actually issue any SQL
             sess.flush()
-        self.assert_sql_count(testbase.db, go, 0)
+        self.assert_sql_count(testing.db, go, 0)
 
     def test_options(self):
         """tests using options on a mapper to create deferred and undeferred columns"""
@@ -913,20 +993,20 @@ class DeferredTest(MapperSuperTest):
         q = sess.query(m)
         q2 = q.options(defer('user_id'))
         def go():
-            l = q2.select()
+            l = q2.all()
             print l[2].user_id
 
-        orderby = str(orders.default_order_by()[0].compile(testbase.db))
-        self.assert_sql(testbase.db, go, [
+        orderby = str(orders.default_order_by()[0].compile(testing.db))
+        self.assert_sql(testing.db, go, [
             ("SELECT orders.order_id AS orders_order_id, orders.description AS orders_description, orders.isopen AS orders_isopen FROM orders ORDER BY %s" % orderby, {}),
             ("SELECT orders.user_id AS orders_user_id FROM orders WHERE orders.order_id = :param_1", {'param_1':3})
         ])
         sess.clear()
         q3 = q2.options(undefer('user_id'))
         def go():
-            l = q3.select()
+            l = q3.all()
             print l[3].user_id
-        self.assert_sql(testbase.db, go, [
+        self.assert_sql(testing.db, go, [
             ("SELECT orders.order_id AS orders_order_id, orders.user_id AS orders_user_id, orders.description AS orders_description, orders.isopen AS orders_isopen FROM orders ORDER BY %s" % orderby, {}),
         ])
 
@@ -940,17 +1020,17 @@ class DeferredTest(MapperSuperTest):
         sess = create_session()
         q = sess.query(m)
         def go():
-            l = q.options(undefer_group('primary')).select()
+            l = q.options(undefer_group('primary')).all()
             o2 = l[2]
             print o2.opened, o2.description, o2.userident
             assert o2.opened == 1
             assert o2.userident == 7
             assert o2.description == 'order 3'
-        orderby = str(orders.default_order_by()[0].compile(testbase.db))
-        self.assert_sql(testbase.db, go, [
+        orderby = str(orders.default_order_by()[0].compile(testing.db))
+        self.assert_sql(testing.db, go, [
             ("SELECT orders.user_id AS orders_user_id, orders.description AS orders_description, orders.isopen AS orders_isopen, orders.order_id AS orders_order_id FROM orders ORDER BY %s" % orderby, {}),
         ])
-        
+
     def test_locates_col(self):
         """test that manually adding a col to the result undefers the column"""
         mapper(Order, orders, properties={
@@ -961,14 +1041,14 @@ class DeferredTest(MapperSuperTest):
         o1 = sess.query(Order).first()
         def go():
             assert o1.description == 'order 1'
-        self.assert_sql_count(testbase.db, go, 1)
-        
+        self.assert_sql_count(testing.db, go, 1)
+
         sess = create_session()
         o1 = sess.query(Order).add_column(orders.c.description).first()[0]
         def go():
             assert o1.description == 'order 1'
-        self.assert_sql_count(testbase.db, go, 0)
-        
+        self.assert_sql_count(testing.db, go, 0)
+
     def test_deepoptions(self):
         m = mapper(User, users, properties={
             'orders':relation(mapper(Order, orders, properties={
@@ -979,19 +1059,19 @@ class DeferredTest(MapperSuperTest):
         })
         sess = create_session()
         q = sess.query(m)
-        l = q.select()
+        l = q.all()
         item = l[0].orders[1].items[1]
         def go():
             print item.item_name
-        self.assert_sql_count(testbase.db, go, 1)
+        self.assert_sql_count(testing.db, go, 1)
         self.assert_(item.item_name == 'item 4')
         sess.clear()
         q2 = q.options(undefer('orders.items.item_name'))
-        l = q2.select()
+        l = q2.all()
         item = l[0].orders[1].items[1]
         def go():
             print item.item_name
-        self.assert_sql_count(testbase.db, go, 0)
+        self.assert_sql_count(testing.db, go, 0)
         self.assert_(item.item_name == 'item 4')
 
 class CompositeTypesTest(ORMTest):
@@ -1075,7 +1155,7 @@ class CompositeTypesTest(ORMTest):
             for e1, e2 in zip(g.edges, g2.edges):
                 assert e1.start == e2.start
                 assert e1.end == e2.end
-        self.assert_sql_count(testbase.db, go, 1)
+        self.assert_sql_count(testing.db, go, 1)
 
         # test comparison of CompositeProperties to their object instances
         g = sess.query(Graph).get([1, 1])
@@ -1133,14 +1213,15 @@ class NoLoadTest(MapperSuperTest):
         q = create_session().query(m)
         l = [None]
         def go():
-            x = q.select(users.c.user_id == 7)
+            x = q.filter(users.c.user_id == 7).all()
             x[0].addresses
             l[0] = x
-        self.assert_sql_count(testbase.db, go, 1)
+        self.assert_sql_count(testing.db, go, 1)
 
         self.assert_result(l[0], User,
             {'user_id' : 7, 'addresses' : (Address, [])},
             )
+
     def test_options(self):
         m = mapper(User, users, properties = dict(
             addresses = relation(mapper(Address, addresses), lazy=None)
@@ -1148,23 +1229,23 @@ class NoLoadTest(MapperSuperTest):
         q = create_session().query(m).options(lazyload('addresses'))
         l = [None]
         def go():
-            x = q.select(users.c.user_id == 7)
+            x = q.filter(users.c.user_id == 7).all()
             x[0].addresses
             l[0] = x
-        self.assert_sql_count(testbase.db, go, 2)
+        self.assert_sql_count(testing.db, go, 2)
 
         self.assert_result(l[0], User,
             {'user_id' : 7, 'addresses' : (Address, [{'address_id' : 1}])},
             )
 
-class MapperExtensionTest(PersistTest):
+class MapperExtensionTest(TestBase):
     def setUpAll(self):
         tables.create()
-        
+
         global methods, Ext
-        
+
         methods = []
-        
+
         class Ext(MapperExtension):
             def load(self, query, *args, **kwargs):
                 methods.append('load')
@@ -1218,10 +1299,10 @@ class MapperExtensionTest(PersistTest):
         clear_mappers()
         methods[:] = []
         tables.delete()
-    
+
     def tearDownAll(self):
         tables.drop()
-            
+
     def test_basic(self):
         """test that common user-defined methods get called."""
         mapper(User, users, extension=Ext())
@@ -1236,17 +1317,17 @@ class MapperExtensionTest(PersistTest):
         sess.flush()
         sess.delete(u)
         sess.flush()
-        self.assertEquals(methods, ['before_insert', 'after_insert', 'load', 'translate_row', 'populate_instance', 'get', 
+        self.assertEquals(methods, ['before_insert', 'after_insert', 'load', 'translate_row', 'populate_instance', 'get',
             'translate_row', 'create_instance', 'populate_instance', 'before_update', 'after_update', 'before_delete', 'after_delete'])
 
     def test_inheritance(self):
         # test using inheritance
         class AdminUser(User):
             pass
-            
+
         mapper(User, users, extension=Ext())
         mapper(AdminUser, addresses, inherits=User)
-        
+
         sess = create_session()
         am = AdminUser()
         sess.save(am)
@@ -1258,17 +1339,17 @@ class MapperExtensionTest(PersistTest):
         sess.flush()
         sess.delete(am)
         sess.flush()
-        self.assertEquals(methods, ['before_insert', 'after_insert', 'load', 'translate_row', 'populate_instance', 'get', 
+        self.assertEquals(methods, ['before_insert', 'after_insert', 'load', 'translate_row', 'populate_instance', 'get',
             'translate_row', 'create_instance', 'populate_instance', 'before_update', 'after_update', 'before_delete', 'after_delete'])
 
     def test_after_with_no_changes(self):
         # test that after_update is called even if no cols were updated
-        
+
         mapper(Item, orderitems, extension=Ext() , properties={
             'keywords':relation(Keyword, secondary=itemkeywords)
         })
         mapper(Keyword, keywords, extension=Ext() )
-        
+
         sess = create_session()
         i1 = Item()
         k1 = Keyword()
@@ -1281,8 +1362,8 @@ class MapperExtensionTest(PersistTest):
         i1.keywords.append(k1)
         sess.flush()
         self.assertEquals(methods, ['before_update', 'after_update'])
-        
-        
+
+
     def test_inheritance_with_dupes(self):
         # test using inheritance, same extension on both mappers
         class AdminUser(User):
@@ -1303,16 +1384,16 @@ class MapperExtensionTest(PersistTest):
         sess.flush()
         sess.delete(am)
         sess.flush()
-        self.assertEquals(methods, ['before_insert', 'after_insert', 'load', 'translate_row', 'populate_instance', 'get', 
+        self.assertEquals(methods, ['before_insert', 'after_insert', 'load', 'translate_row', 'populate_instance', 'get',
             'translate_row', 'create_instance', 'populate_instance', 'before_update', 'after_update', 'before_delete', 'after_delete'])
-        
-class RequirementsTest(AssertMixin):
+
+class RequirementsTest(TestBase, AssertsExecutionResults):
     """Tests the contract for user classes."""
 
     def setUpAll(self):
         global metadata, t1, t2, t3, t4, t5, t6
 
-        metadata = MetaData(testbase.db)
+        metadata = MetaData(testing.db)
         t1 = Table('ht1', metadata,
                    Column('id', Integer, primary_key=True),
                    Column('value', String(10)))
@@ -1439,7 +1520,7 @@ class RequirementsTest(AssertMixin):
 
         s.flush()
         self.assertEquals(t1.count().scalar(), 4)
-        
+
         h6 = H6()
         h6.h1a = h1
         h6.h1b = h1
@@ -1448,7 +1529,7 @@ class RequirementsTest(AssertMixin):
         h6.h1a = h1
         h6.h1b = x = H1()
         assert x in s
-        
+
         h6.h1b.h2s.append(H2())
 
         s.flush()
@@ -1478,4 +1559,4 @@ class RequirementsTest(AssertMixin):
 
 
 if __name__ == "__main__":
-    testbase.main()
+    testenv.main()
