@@ -1,4 +1,4 @@
-import testbase
+import testenv; testenv.configure_for_tests()
 import unittest
 
 
@@ -8,19 +8,20 @@ def suite():
         'sql.constraints',
 
         'sql.generative',
-        
+
         # SQL syntax
         'sql.select',
         'sql.selectable',
-        'sql.case_statement', 
+        'sql.case_statement',
         'sql.labels',
         'sql.unicode',
-        
+
         # assorted round-trip tests
+        'sql.functions',
         'sql.query',
         'sql.quote',
         'sql.rowcount',
-        
+
         # defaults, sequences (postgres/oracle)
         'sql.defaults',
         )
@@ -33,4 +34,4 @@ def suite():
     return alltests
 
 if __name__ == '__main__':
-    testbase.main(suite())
+    testenv.main(suite())
