@@ -1,4 +1,5 @@
-import testbase
+# can't be imported until the path is setup; be sure to configure
+# first if covering.
 from sqlalchemy import *
 from sqlalchemy import util
 from testlib import *
@@ -8,62 +9,74 @@ class Base(object):
     def __init__(self, **kwargs):
         for k in kwargs:
             setattr(self, k, kwargs[k])
-    
+
     # TODO: add recursion checks to this
     def __repr__(self):
         return "%s(%s)" % (
-            (self.__class__.__name__), 
+            (self.__class__.__name__),
             ','.join(["%s=%s" % (key, repr(getattr(self, key))) for key in self.__dict__ if not key.startswith('_')])
         )
-    
+
     def __ne__(self, other):
         return not self.__eq__(other)
-        
+
     def __eq__(self, other):
         """'passively' compare this object to another.
-        
+
         only look at attributes that are present on the source object.
-        
+
         """
 
         if self in _recursion_stack:
             return True
         _recursion_stack.add(self)
         try:
-            # use __dict__ to avoid instrumented properties
-            for attr in self.__dict__.keys():
+            # pick the entity thats not SA persisted as the source
+            if other is None:
+                a = self
+                b = other
+            elif hasattr(self, '_instance_key'):
+                a = other
+                b = self
+            else:
+                a = self
+                b = other
+
+            for attr in a.__dict__.keys():
                 if attr[0] == '_':
                     continue
-                value = getattr(self, attr)
+                value = getattr(a, attr)
+                #print "looking at attr:", attr, "start value:", value
                 if hasattr(value, '__iter__') and not isinstance(value, basestring):
                     try:
                         # catch AttributeError so that lazy loaders trigger
-                        otherattr = getattr(other, attr)
+                        battr = getattr(b, attr)
                     except AttributeError:
-                        #print "Other class does not have attribute named '%s'" % attr
+                        #print "b class does not have attribute named '%s'" % attr
                         return False
+                    #print "other:", battr
                     if not hasattr(value, '__len__'):
                         value = list(iter(value))
-                        otherattr = list(iter(otherattr))
-                    if len(value) != len(otherattr):
-                        #print "Length of collection '%s' does not match that of other" % attr
+                        battr = list(iter(battr))
+                    if len(value) != len(battr):
+                        #print "Length of collection '%s' does not match that of b" % attr
                         return False
-                    for (us, them) in zip(value, otherattr):
+                    for (us, them) in zip(value, battr):
                         if us != them:
-                            #print "1. Attribute named '%s' does not match other" % attr
+                            #print "1. Attribute named '%s' does not match b" % attr
                             return False
                     else:
                         continue
                 else:
                     if value is not None:
-                        if value != getattr(other, attr, None):
-                            #print "2. Attribute named '%s' does not match that of other" % attr
+                        if value != getattr(b, attr, None):
+                            #print "2. Attribute named '%s' does not match that of b" % attr
                             return False
             else:
                 return True
         finally:
             _recursion_stack.remove(self)
-            
+
 class User(Base):pass
 class Order(Base):pass
 class Item(Base):pass
@@ -85,18 +98,18 @@ orders = Table('orders', metadata,
     Column('isopen', Integer)
     )
 
-addresses = Table('addresses', metadata, 
+addresses = Table('addresses', metadata,
     Column('id', Integer, primary_key=True),
     Column('user_id', None, ForeignKey('users.id')),
     Column('email_address', String(50), nullable=False))
 
-dingalings = Table("dingalings", metadata, 
+dingalings = Table("dingalings", metadata,
     Column('id', Integer, primary_key=True),
     Column('address_id', None, ForeignKey('addresses.id')),
     Column('data', String(30))
     )
-    
-items = Table('items', metadata, 
+
+items = Table('items', metadata,
     Column('id', Integer, primary_key=True),
     Column('description', String(30), nullable=False)
     )
@@ -105,11 +118,11 @@ order_items = Table('order_items', metadata,
     Column('item_id', None, ForeignKey('items.id')),
     Column('order_id', None, ForeignKey('orders.id')))
 
-item_keywords = Table('item_keywords', metadata, 
+item_keywords = Table('item_keywords', metadata,
     Column('item_id', None, ForeignKey('items.id')),
     Column('keyword_id', None, ForeignKey('keywords.id')))
 
-keywords = Table('keywords', metadata, 
+keywords = Table('keywords', metadata,
     Column('id', Integer, primary_key=True),
     Column('name', String(30), nullable=False)
     )
@@ -138,7 +151,7 @@ def install_fixture_data():
         dict(id = 2, user_id = 9, description = 'order 2', isopen=0, address_id=4),
         dict(id = 3, user_id = 7, description = 'order 3', isopen=1, address_id=1),
         dict(id = 4, user_id = 9, description = 'order 4', isopen=1, address_id=4),
-        dict(id = 5, user_id = 7, description = 'order 5', isopen=0, address_id=1)
+        dict(id = 5, user_id = 7, description = 'order 5', isopen=0, address_id=None)
     )
     items.insert().execute(
         dict(id=1, description='item 1'),
@@ -177,7 +190,7 @@ def install_fixture_data():
 
     # this many-to-many table has the keywords inserted
     # in primary key order, to appease the unit tests.
-    # this is because postgres, oracle, and sqlite all support 
+    # this is because postgres, oracle, and sqlite all support
     # true insert-order row id, but of course our pal MySQL does not,
     # so the best it can do is order by, well something, so there you go.
     item_keywords.insert().execute(
@@ -194,35 +207,35 @@ def install_fixture_data():
 
 class FixtureTest(ORMTest):
     refresh_data = False
-    
+
     def setUpAll(self):
         super(FixtureTest, self).setUpAll()
         if self.keep_data:
             install_fixture_data()
-    
+
     def setUp(self):
         if self.refresh_data:
             install_fixture_data()
-            
+
     def define_tables(self, meta):
         pass
 FixtureTest.metadata = metadata
-    
+
 class Fixtures(object):
     @property
     def user_address_result(self):
         return [
             User(id=7, addresses=[
                 Address(id=1)
-            ]), 
+            ]),
             User(id=8, addresses=[
                 Address(id=2, email_address='ed@wood.com'),
                 Address(id=3, email_address='ed@bettyboop.com'),
                 Address(id=4, email_address='ed@lala.com'),
-            ]), 
+            ]),
             User(id=9, addresses=[
                 Address(id=5)
-            ]), 
+            ]),
             User(id=10, addresses=[])
         ]
 
@@ -235,18 +248,18 @@ class Fixtures(object):
                 Order(description='order 1', items=[Item(description='item 1'), Item(description='item 2'), Item(description='item 3')]),
                 Order(description='order 3'),
                 Order(description='order 5'),
-            ]), 
+            ]),
             User(id=8, addresses=[
                 Address(id=2),
                 Address(id=3),
                 Address(id=4)
-            ]), 
+            ]),
             User(id=9, addresses=[
                 Address(id=5)
             ], orders=[
                 Order(description='order 2', items=[Item(description='item 1'), Item(description='item 2'), Item(description='item 3')]),
                 Order(description='order 4', items=[Item(description='item 1'), Item(description='item 5')]),
-            ]), 
+            ]),
             User(id=10, addresses=[])
         ]
 
@@ -264,8 +277,8 @@ class Fixtures(object):
                 Order(id=4, items=[Item(id=1), Item(id=5)]),
             ]),
             User(id=10)
-        ] 
-    
+        ]
+
     @property
     def item_keyword_result(self):
         return [
